# -*- coding: UTF8 -*-
# vim: ts=4 noet

import sys, os, string, quopri, codecs, base64, gc
import rox
from rox import g, filer, app_options, AppInfo, InfoWin, OptionsBox, Menu, mime, basedir
from rox.loading import XDSLoader
from rox.saving import Saveable, SaveBox
from rox.options import Option
import gobject
import widgets
import EContact 


APP_NAME=('Contacts')
TITLE_DEFAULT = _("New Contact")

from webbrowser import _iscommand as iscommand
if iscommand('rox'):
	filemanager = 'rox'
elif iscommand('nautilus'):
	filemanager = 'nautilus'
elif iscommand('xffm'):
	filemanager = 'xffm'
else:
	filemanager = 'x11'
	
def roxUI(): return filemanager == 'rox'

try:
	mime_type = rox.mime.get_type_by_name('dummy.vcf')
	MIME_TYPE_VCARD = mime_type.media + '/' + mime_type.subtype
except:
	MIME_TYPE_VCARD = 'text/directory'

known_codecs = (
	"iso8859_1", "iso8859_2", "iso8859_3", "iso8859_4", "iso8859_5",
	"iso8859_6", "iso8859_7", "iso8859_8", "iso8859_9", "iso8859_10",
	"iso8859_13", "iso8859_14", "iso8859_15",
	"ascii", "base64_codec", "charmap",
	"cp037", "cp1006", "cp1026", "cp1140", "cp1250", "cp1251", "cp1252",
	"cp1253", "cp1254", "cp1255", "cp1256", "cp1257", "cp1258", "cp424",
	"cp437", "cp500", "cp737", "cp775", "cp850", "cp852", "cp855", "cp856",
	"cp857", "cp860", "cp861", "cp862", "cp863", "cp864", "cp865", "cp866",
	"cp869", "cp874", "cp875", "hex_codec",
	"koi8_r",
	"latin_1",
	"mac_cyrillic", "mac_greek", "mac_iceland", "mac_latin2", "mac_roman", "mac_turkish",
	"mbcs", "quopri_codec", "raw_unicode_escape",
	"rot_13",
	"utf_16_be", "utf_16_le", "utf_16", "utf_7", "utf_8", "uu_codec",
	"zlib_codec"
)

foldmarks = ['\r\n\t', '\r\n ',  '\n\t', '\n ']
foldmark = '\n '

tel_options = [ _('Landline'), _('ISDN'), _('Mobile'), _('Car'), _('Video'),
		_('Pager'), _('Fax'), _('Modem'), _('BBS'), _('PCS') ]
tel_types = [ 'VOICE', 'ISDN', 'CELL', 'CAR', 'VIDEO',
		'PAGER', 'FAX', 'MODEM', 'BBS', 'PCS' ]

im_options = [ 'AIM', 'Gadu-Gadu', 'GroupWise', 'ICQ', 'IRC', 'Jabber',
		'MSN', 'Napster', 'Yahoo', 'Zephyr' ]
im_types = [ 'X-AIM', 'X-GADU-GADU', 'X-GROUPWISE', 'X-ICQ', 'X-IRC', 'X-JABBER',
		'X-MSN', 'X-NAPSTER', 'X-YAHOO', 'X-ZEPHYR' ]

def utc_offset(tz):
	import time
	os.environ['TZ'] = tz ; time.tzset()
	(x, y) = divmod(time.timezone, 3600)
	if x >=0: h = "%2d" % x
 	else: h = ("-%2d" % abs(x))
	if y >=0: m = "%2d" % y
	else: m = ("-%2d" % abs(y))
	return ("%s:%s" % (h, m)).replace(' ', '0')
	
class Entries:
	pass
	
class Birthdate:
	year = None
	month = None
	day = None
	string = None
	checked = False

class ImageProperty:
	
	def __init__(self, src = None):
		if src:
			self.mime_type = src.mime_type
			self.data = src.data
			self.pixbuf = src.pixbuf
			self.b64 = src.b64
			self.uri = src.uri
		else:	
			self.mime_type = None
			self.data = None
			self.pixbuf = None
			self.b64 = None
			self.uri = None

	def copy(self, s, d):
		d.mime_type = s.mime_type
		d.data = s.data
		d.pixbuf = s.pixbuf
		d.b64 = s.b64
		d.uri = s.uri

	def from_data(self, data):
		loader = g.gdk.PixbufLoader()
		loader.write(data)
		loader.close()
		self.pixbuf = loader.get_pixbuf()
		self.mime_type = loader.get_format()['mime_types'][0]
		self.data = data
		self.b64 = base64.encodestring(data)
		
	def from_stream(self, stream):
		self.from_data(stream.read())
			
	def from_base64(self, b64):
		self.from_data(base64.decodestring(b64))

	def from_file(self, filename):
		file = open(filename, 'r')
		self.from_stream(file)
		file.close()

	def from_uri(self, uri):
		self.uri = uri
		# TODO: download from uri and call self.from_data()
	
def register_iconsets(icon_info):
	iconfactory = g.IconFactory()	
	theme = g.icon_theme_get_default()
	iconinfo = theme.lookup_icon('stock_timezone', 48, 0)
	if iconinfo:
		iconfactory.add('rox-stock-timezone', g.IconSet(iconinfo.load_icon()))	
	iconinfo = theme.lookup_icon('stock_notes', 48, 0)
	if iconinfo:
		iconfactory.add('rox-stock-notes', g.IconSet(iconinfo.load_icon()))
	iconinfo = theme.lookup_icon('stock_effects-sound.png', 48, 0)
	if iconinfo:
		iconfactory.add('rox-stock-sound', g.IconSet(iconinfo.load_icon()))
	image_path = rox.app_dir+'/images/'
	stock_ids = g.stock_list_ids()
	for stock_id, file in icon_info:
		# only load image files when our stock_id is not present
		if stock_id not in stock_ids:
			pixbuf = g.gdk.pixbuf_new_from_file(image_path+file)
			iconset = g.IconSet(pixbuf)
			iconfactory.add(stock_id, iconset)
	iconfactory.add_default()

class ContactWindow(rox.Window, XDSLoader, Saveable):
#<toolitem name="Search" action="Search"/>
	ui = '''<ui>
    <toolbar name="Toolbar">
      <toolitem name="New" action="New"/>
	  <toolitem name="Clear" action="Clear"/>
      <toolitem name="Show" action="Show"/>
      <toolitem name="Save" action="Save"/>
	  <toolitem name="SaveAs" action="SaveAs"/>
	  <separator/>
      <toolitem name="Options" action="Options"/>
	  <toolitem name="Help" action="Help"/>
	  <separator/>
	  <placeholder name="searchbar"/>
    </toolbar>
    </ui>'''

	register_iconsets([
					('rox-stock-timezone', 'stock_timezone.png'),
					('rox-stock-notes', 'stock_notes.png'),
					('rox-stock-sound', 'sound.png'),
					('rox-contacts-blue-key', 'blue_key.xpm'),
					('rox-contacts-show-stack', 'stack-of-cards.xpm'),
				   	('rox-contacts-edit-privacy', 'privacy.png'),
				   	('rox-contacts-cake-white', 'cake_white.png'),
				   	('rox-contacts-cake-yellow', 'cake_yellow.png'),
				   	('rox-contacts-cake-green', 'cake_green.png'),
					('rox-contacts-cake-red', 'cake_red.png'),	
				   	('rox-contacts-photo', 'photo.png'),
				   	('rox-contacts-logo', 'logo.png')
	]) 
		
	entry = None
	revert = None	
	rox.setup_app_options('Contacts')
	card_folder = Option('card_folder', os.path.expanduser('~')+'/Contacts')
	file_as = Option('file_as', 'reverse_comma')
	mailto_handler = Option('mailto_handler', '')
	im_handler = Option('im_handler', '')
	url_handler = Option('url_handler', '')
	callto_handler = Option('callto_handler', '')
	audio_handler = Option('audio_handler', '')
	rox.app_options.notify()

	def __init__(self, filename = None):
		rox.Window.__init__(self)
		XDSLoader.__init__(self, [MIME_TYPE_VCARD, 'UTF8_STRING'])

		self.mime_type_vcard = mime.lookup(MIME_TYPE_VCARD)

		self.connect('destroy', self.destroyed)
		self.connect('delete-event', self.delete_event)
		rox.app_options.add_notify(self.options_changed)
		
		if self.audio_handler.value == '':
			if iscommand('playsound'):
				self.audio_handler.value = 'playsound'
			elif iscommand('aplay'):
				self.audio_handler.value = 'aplay'
			elif roxUI():
				self.audio_handler.value = filemanager
		
		self.savebox = None
		self.info_box = None
		self.revert = {}
		self.revert_bday = None
		self.revert_key = None
		self.revert_classification = 'PUBLIC'
		self.revert_tz = None
		self.revert_geo = None
		self.revert_sound = None
		self.revert_photo = None
		self.revert_logo = None

		#self.ecard = EContact.ECard()	
		#self.ecard = EContact.Properties()

		self.entry = Entries()

		self.uuid = ''
		self.note_text = ''
		self.classification = 'PUBLIC'
		self.tz = None
		self.geo = None
		self.key = None
		self.sound = None
		self.photo = None
		self.logo = None
		self.fileas = None
		self.rev = None
		self.contactname = EContact.Name()
		self.learned_names = EContact.Name()
		self.learned_names.prefixes = EContact.NamePrefix
		self.learned_names.suffixes = EContact.NameSuffix
		
		self.unknown_properties = {}

		self.contactbox = g.VBox(False, spacing = 4)
		self.add(self.contactbox)
		self.tooltips = g.Tooltips()

		self.uimanager = g.UIManager()
		self.uimanager.ensure_update()
		self.add_accel_group(self.uimanager.get_accel_group())
		self.actiongroup = g.ActionGroup('UIManager')

		self.build_menu()
		self.build_toolbar()
		
		gc.collect()
		
		# prepare notebook container
		self.notebook =  g.Notebook() ; self.contactbox.add(self.notebook)
		self.sizegroup_notebook = g.SizeGroup(g.SIZE_GROUP_HORIZONTAL)
		self.sizegroup_option_entry = g.SizeGroup(g.SIZE_GROUP_HORIZONTAL)

		# add default notebook tabs
		self.build_notebook_home(self.notebook, self.sizegroup_notebook, self.sizegroup_option_entry)
		self.build_notebook_work(self.notebook, self.sizegroup_notebook, self.sizegroup_option_entry)

		for item in self.entry.__dict__.values():
			item.connect('changed', self.update_discard)

		self.show_all()
		
		if filename != None:
			self.uri = os.path.abspath(filename)
			self.load(self.uri)
			if self.get_title() == self.entry.org.get_text():
				self.notebook.set_current_page(1)
		else:	
			self.update_title()

		self.search()

		self.init_discard()

		
	def init_discard(self):
		for item in self.entry.__dict__.values():
			self.revert[item] = item.get_text()
		self.bday.string = self.bday_to_string()
		self.revert_bday = self.bday.string
		self.revert_sound = self.sound
		self.revert_key = self.key
		self.revert_classification = self.classification
		self.revert_geo = self.geo
		self.revert_tz = self.tz

		if self.revert_photo:
			del self.revert_photo
		
		if self.photo:
			self.revert_photo = ImageProperty(self.photo)
		else:
			self.revert_photo = None
			
		if self.revert_logo:
			del self.revert_logo
			
		if self.logo:
			self.revert_logo = ImageProperty(self.logo)
		else:
			self.revert_logo = None
		
		self.toggle_birthdate(self.bday.checked)
				
	def build_toolbar(self):
		self.actiongroup.add_actions([('New', g.STOCK_NEW, _('New Card'), None, _('New Card'), self.new),
				('Clear', g.STOCK_CLEAR, _('Clear Card'), None, _('Clear Card'), self.clear),
				('Show', 'rox-contacts-show-stack',_('Show Cards'), None, _('Show Cards'), self.show),
				('Save', g.STOCK_SAVE, _('Save Card'), None, _('Save Card'), self.save),
				('SaveAs', g.STOCK_SAVE_AS, _('Save Card As')+'...', None, _('Save Card As')+'...', self.saveas),
				('Options', g.STOCK_PREFERENCES, _('Edit Options'), None, _('Edit Options'), self.show_options),
				('Help', g.STOCK_HELP, _('Show Help Files'), None, _('Show Help Files'), self.help),
				('Note', 'rox-stock-notes', _('Take Note'), None, _('Take Note'), self.edit_note),
				('Privacy', 'rox-contacts-edit-privacy', _('Privacy'), None, _('Privacy'), self.edit_privacy),
				('Geo', 'rox-stock-timezone', _('Geo Data'), None, _('Geo Data'), self.edit_geo)	
		])
		self.uimanager.insert_action_group(self.actiongroup, 0)
		self.uimanager.add_ui_from_string(self.ui)
		self.toolbar = self.uimanager.get_widget('/Toolbar')
		hbox = g.HBox(False)
		hbox.pack_start(self.toolbar, True, True, 0)
		self.contactbox.pack_start(hbox, False, False, 0)

		self.entry_search = g.Entry()
		self.search_label = g.Label(_('Search')+': ')
		
		#self.search_label.set_size_request(40, -1)
		self.search_label.unset_flags(g.CAN_FOCUS)
		self.entry_search.set_width_chars(14)
		self.searchbar = g.ToolItem()
		hbox = g.HBox(False) ; self.searchbar.add(hbox)
		hbox.pack_start(self.search_label, False, False, 0)
		hbox.pack_start(self.entry_search, False, False, 0)
		#self.searchbar.set_expand(True) 
		completion = g.EntryCompletion()
		self.liststore = g.ListStore(str)
		completion.set_model(self.liststore)
		self.entry_search.set_completion(completion)
		completion.set_text_column(0)
		completion.connect('match-selected', self.match)

		#spacer = g.ToolItem()
		self.toolbar.insert(g.SeparatorToolItem(), -1)
		self.toolbar.insert(g.ToolItem(), -1)
		#self.toolbar.insert(g.ToolItem(), -1)
		self.toolbar.insert(self.searchbar, -1)

		
		
	def build_menu(self):
		self.add_events(g.gdk.BUTTON_PRESS_MASK)
		self.connect('button-press-event', self.button_press)
		Menu.set_save_name(APP_NAME)
		self.menu = Menu.Menu('main', [ 
			Menu.Action(_('New'),	'new',		'<Ctrl>N', g.STOCK_NEW),
			Menu.Action(_('Clear'),	'clear',	'<Ctrl>D', g.STOCK_CLEAR),
			Menu.Action(_('Delete'), 'delete',	'<Shift><Ctrl>D', g.STOCK_DELETE),
			Menu.Action(_('Save'),	'save',		'<Ctrl>S', g.STOCK_SAVE),
			Menu.Action(_('Save As'), 'saveas',	'<Shift><Ctrl>S', g.STOCK_SAVE_AS),
			Menu.Action(_('Show Cards'),	'show',	'F2', 'rox-contacts-show-stack'),
			Menu.SubMenu(_('My Card')+'...', [
				Menu.Action(_('Open my Card'),	'find_me',	'<Alt>M', None),
				Menu.Action(_('That\'s my Card'),	'me',	'', None),]),
			Menu.Action(_('Options'),		'show_options', '', g.STOCK_PREFERENCES),
			Menu.Action(_('Note'),	'edit_note', '<Alt>N', "rox-stock-notes"),
			Menu.Action(_('Privacy'), 'edit_privacy', '<Alt>P', "rox-contacts-edit-privacy"),
			Menu.Action(_('Geo'), 'edit_geo', '<Alt>G', "rox-stock-timezone"),
			Menu.Action(_('Help'),		'help',		'F1', g.STOCK_HELP),
			Menu.Separator(),
			Menu.Action(_('Close'),	'close',	'<Ctrl>W', g.STOCK_CLOSE)])				
		self.menu.attach(self,self)

	def build_notebook_home(self, notebook, sg, sizegroup_option_entry):
		home_box = g.VBox(False, spacing = 4)
		notebook.append_page(home_box, g.Label(_('Home')))
		
		notebook.set_show_tabs(True)

        # NAME: FN: N:
		hbox = g.HBox(False) ; home_box.pack_start(hbox, False, False, 1)
		(self.entry.name, self.button_name) = \
			widgets.edit_button(hbox, sg, _('Name'))   
		self.button_name.connect('clicked', self.edit_name)
		self.tooltips.set_tip(self.button_name,  _('Click to open Name Editing Dialog.')\
				+'\n'+_('Use to assign or check name components.'))

		
		hbox = g.HBox(False) ; home_box.pack_start(hbox, False, False, 0)
		self.entry.nick = widgets.entry_with_label(hbox, sg, _('Nickname'))
		self.tooltips.set_tip(self.entry.nick,  _('The nickname is the descriptive name given instead of or in addition to the one belonging to a person, place, or thing. It can also be used to specify a familiar form of a proper name.'))

		# Birthdate (BDAY)
		self.bday = Birthdate()
		(bday_box, self.bday.checked, self.bday.day, \
		self.bday.month, self.bday.year) = \
						widgets.date_entry(sg, _("Birthdate")+(": "), self.tooltips) 
		self.bday.checked.connect('toggled', self.toggle_birthdate)
		self.bday.year.connect('changed', self.update_discard)
		self.bday.month.connect('changed', self.update_discard)
		self.bday.day.connect('changed', self.update_discard)
		hbox.pack_start(bday_box, False, False, 3)
	 
		# Address (ADR)
		hbox = g.HBox(False) ; home_box.pack_start(hbox, True, True, 0)
		self.entry.street = widgets.entry_with_label(hbox, sg, _('Street'))
		self.entry.city = widgets.entry_with_label(hbox, sg, _('City'))
		hbox = g.HBox(False) ; home_box.pack_start(hbox, True, True, 0)
		self.entry.state = widgets.entry_with_label(hbox, sg, _('State'))
		self.entry.zip = widgets.entry_with_label(hbox, sg, _('Postal Code'))
		hbox = g.HBox(False) ; home_box.pack_start(hbox, True, True, 0)
		self.entry.country = widgets.entry_with_label(hbox, sg, _('Country'))
		self.entry.pobox = widgets.entry_with_label(hbox, sg, _('P.O. Box'))   
	   
		# CATEGORIES
		hbox = g.HBox(False) ; home_box.pack_start(hbox, True, True, 0)
		self.entry.categories = widgets.entry_with_label(hbox, sg, _('Categories'))
		self.tooltips.set_tip(self.entry.categories, _('Comma separated list of search categories'))
			 
		# Phone (TEL)
		widgets.section_header(home_box, _('Telephone'))
		
		hbox = g.HBox(False) ; home_box.pack_start(hbox, True, True, 3)
		(self.entry.tel_1, self.type_tel_1) = widgets.option_entry(hbox, sizegroup_option_entry, tel_options, 0)
		(self.entry.tel_2, self.type_tel_2) = widgets.option_entry(hbox, sizegroup_option_entry, tel_options, 1)
		hbox = g.HBox(False) ; home_box.pack_start(hbox, True, True, 3)
		(self.entry.tel_3, self.type_tel_3) = widgets.option_entry(hbox, sizegroup_option_entry, tel_options, 2)
		(self.entry.tel_4, self.type_tel_4) = widgets.option_entry(hbox, sizegroup_option_entry, tel_options, 6)
		

		# Instant Messaging
		widgets.section_header(home_box, _('Instant Messaging'))

		hbox = g.HBox(False) ; home_box.pack_start(hbox, True, True, 3)
		(self.entry.im_1, self.type_im_1) = widgets.option_entry_with_icon(hbox, im_options, 0) 
		(self.entry.im_2, self.type_im_2) = widgets.option_entry_with_icon(hbox, im_options, 3)
		hbox = g.HBox(False) ; home_box.pack_start(hbox, True, True, 3)
		(self.entry.im_3, self.type_im_3) = widgets.option_entry_with_icon(hbox, im_options, 4)
		(self.entry.im_4, self.type_im_4) = widgets.option_entry_with_icon(hbox, im_options, 5)

		home_box.pack_start(g.EventBox(), True, True, 3) # spacer

		# Email (EMAIL;TYPE=INTERNET,HOME)
		hbox = g.HBox(False) ; home_box.pack_start(hbox, True, True, 3)
		(self.entry.email_1, self.button_email_1) = \
				widgets.href_button(hbox, sg, _('Email')+' 1')
		self.button_email_1.connect('clicked', self.launch_mailto_home_1)
		(self.entry.email_2, self.button_email_2) = \
				widgets.href_button(hbox, sg, _('Email')+' 2')
		self.button_email_2.connect('clicked', self.launch_mailto_home_2)
		
		# Homepage (URL:TYPE=HOME)
		(self.entry.url, self.button_url) = \
			widgets.href_button(home_box, sg, _('Web Page'))   
		self.button_url.connect('clicked', self.launch_url_home)

		# Video-Conference (URL:TYPE=HOME)
		(self.entry.videoconference, self.button_videoconference) = \
			widgets.href_button(home_box, sg, _('Video-Conference'))
		self.button_videoconference.connect('clicked', self.launch_callto_home)

	def build_notebook_work(self, notebook, sg, sizegroup_option_entry):
		work_box = g.VBox(False, spacing = 4)
		notebook.append_page(work_box, g.Label(_('Work')))

		self.entry.org = widgets.entry_with_label(work_box, sg, _('Organization'))
		self.tooltips.set_tip(self.entry.org,  _('Organizational Name and associated Units'))
		# Address (ADR)
		hbox = g.HBox(False) ; work_box.pack_start(hbox, True, True, 0)
		self.entry.work_street =  widgets.entry_with_label(hbox, sg, _('Street'))
		self.entry.work_city = widgets.entry_with_label(hbox, sg, _('City'))
		hbox = g.HBox(False) ; work_box.pack_start(hbox, True, True, 0)
		self.entry.work_state = widgets.entry_with_label(hbox, sg, _('State'))
		self.entry.work_zip = widgets.entry_with_label(hbox, sg, _('Postal Code'))
		hbox = g.HBox(False) ; work_box.pack_start(hbox, True, True, 0)
		self.entry.work_country = widgets.entry_with_label(hbox, sg, _('Country'))
		self.entry.work_pobox = widgets.entry_with_label(hbox, sg, _('P.O. Box'))
		
		self.entry.jobtitle = widgets.entry_with_label(work_box, sg, _('Job Title'))
		self.tooltips.set_tip(self.entry.jobtitle, _('Job Title, Functional Position or Function'))
		self.entry.role = widgets.entry_with_label(work_box, sg, _('Occupation'))
		self.tooltips.set_tip(self.entry.role, _('Role, Occupation or Business Category'))

		# Phone (TEL)
		widgets.section_header(work_box, _('Telephone'))
		
		hbox = g.HBox(False) ; work_box.pack_start(hbox, True, True, 3)
		(self.entry.work_tel_1, self.type_work_tel_1) = widgets.option_entry(hbox, sizegroup_option_entry, tel_options, 0)
		(self.entry.work_tel_2, self.type_work_tel_2) = widgets.option_entry(hbox, sizegroup_option_entry, tel_options, 1)
		hbox = g.HBox(False) ; work_box.pack_start(hbox, True, True, 3)
		(self.entry.work_tel_3, self.type_work_tel_3) = widgets.option_entry(hbox, sizegroup_option_entry, tel_options, 2)
		(self.entry.work_tel_4, self.type_work_tel_4) = widgets.option_entry(hbox, sizegroup_option_entry, tel_options, 6)

		# Instant Messaging
		widgets.section_header(work_box, _('Instant Messaging'))

		hbox = g.HBox(False) ; work_box.pack_start(hbox, True, True, 3)
		(self.entry.work_im_1, self.type_work_im_1) = widgets.option_entry_with_icon(hbox, im_options, 0) 
		(self.entry.work_im_2, self.type_work_im_2) = widgets.option_entry_with_icon(hbox, im_options, 3)
		hbox = g.HBox(False) ; work_box.pack_start(hbox, True, True, 3)
		(self.entry.work_im_3, self.type_work_im_3) = widgets.option_entry_with_icon(hbox, im_options, 4)
		(self.entry.work_im_4, self.type_work_im_4) = widgets.option_entry_with_icon(hbox, im_options, 5)

		work_box.pack_start(g.EventBox(), True, True, 3) # spacer
	   
	   # Email (EMAIL;TYPE=INTERNET,WORK)
		(self.entry.work_email, self.button_work_email) = \
			widgets.href_button(work_box, sg, _('Email'))
		self.button_work_email.connect('clicked', self.launch_mailto_work)
		
		# Homepage (URL:TYPE=WORK)
		(self.entry.work_url, self.button_work_url) = \
			widgets.href_button(work_box, sg, _('Web Page'))   
		self.button_work_url.connect('clicked', self.launch_url_work)

		# Video-Conference (URL:TYPE=WORK)
		(self.entry.work_videoconference, self.button_work_videoconference) = \
			widgets.href_button(work_box, sg, _('Video-Conference'))
		self.button_work_videoconference.connect('clicked', self.launch_callto_work)


	def button_press(self, text, event):
		"""Popup menu handler"""
		if event.button != 3:
			return 0
		self.menu.popup(self, event)
		return 1	
		
	def toggle_birthdate(self, widget):
		"""Enable/Disable Birthdate Widget"""
		flag = widget.get_active()
		if flag:
			self.bday.string = self.bday_to_string()
		else:
			self.bday.string = ''
		self.bday.day.set_sensitive(flag)
		self.bday.month.set_sensitive(flag)
		self.bday.year.set_sensitive(flag)			
		self.update_discard()
		
	# Toolbar Callbacks
	def new(self, widget = None):
		"""New Contact Form"""
		ContactWindow()
		 
	def help(self, button = None):
		"""Show Help Files"""
		helpdir = os.path.join(rox.app_dir, 'Help')
		if roxUI():
			filer.open_dir(helpdir)
		elif filemanager == 'nautilus':
			self.command([filemanager,  helpdir ])
		elif filemanager == 'xffm':
			self.command([filemanager,  helpdir ])
			
	def edit_geo(self, widget = None):
		"""Geo properties dialog"""
		dialog = g.Dialog(_('Edit Geo Data'), self, g.DIALOG_DESTROY_WITH_PARENT)
		dialog.add_button(g.STOCK_CANCEL, g.RESPONSE_CANCEL)
		dialog.add_button(g.STOCK_OK, g.RESPONSE_OK)
		dialog.set_default_response(g.RESPONSE_OK)
		dialog.vbox.set_spacing(4)
		dialog.vbox.set_homogeneous(False)
		
		import geodialog
		timezone_widget = geodialog.Dialog(tz=self.tz, geo=self.geo)
		dialog.vbox.add(timezone_widget)
	
		dialog.show_all()
		
		response = dialog.run()

		if response == g.RESPONSE_OK:
			timezone = timezone_widget.getCurrent()
			import time
			self.tz = [utc_offset(timezone.tz), time.tzname[0], timezone.tz]	
			if timezone_widget.geocheckmark.get_active():
				self.geo = [timezone_widget.latitude.get_value(), 
							timezone_widget.longitude.get_value()]
			if self.entry.country.get_text() == '':
				self.entry.country.set_text(timezone.code)
			self.update_discard()
			
		dialog.destroy()
			
	def edit_privacy(self, widget = None):
		"""Privacy properties dialog"""
		dialog = g.Dialog(_('Edit Privacy'), self, g.DIALOG_DESTROY_WITH_PARENT)
		dialog.add_button(g.STOCK_DELETE, g.RESPONSE_NO)
		dialog.add_button(g.STOCK_OK, g.RESPONSE_OK)	
		dialog.set_default_response(g.RESPONSE_OK)
		dialog.vbox.set_spacing(4)
		dialog.vbox.set_homogeneous(False)
		
		def combo_changed(widget=None):
			active = combo.get_active()
			button_import.set_sensitive(False)
			button_save.set_sensitive(False)
			if active >=0:
				keyID = gpg_keys[active]
				if keyID:
					button_save.set_sensitive(True)
					if newkey and keyID == newkey.keyID:
						button_import.set_sensitive(True)
						self.key = newkey
					elif not self.key or keyID != self.key.keyID:
						try:
							key = gpg.Key()
							key.keyID = keyID
							key.UserID = model[active][0]
							#FIXME: why does this always return the same key?
							#key.value = gpg.MyGnuPG().get_value(keyID)
							key.value = gpg.get_value(keyID)
							self.key = key
						except:
							self.key = newkey
			self.update_discard()
			
		widgets.section_header(dialog.vbox, _('PGP Key'))
		
		combo = g.combo_box_new_text()
		model = combo.get_model()	
		dialog.vbox.pack_start(combo, True, True, 0)
		combo.connect('changed', combo_changed)
				
		def import_key(widget):
			try:
				gpg.MyGnuPG().merge(newkey.value)
				newkey = None
				widget.set_active(False)
			except:
				rox.info(_('Importing Key failed!'))	
				
		class SaveKeyBox(SaveBox):
			def build_main_area(self):
				key = self.save_area.document.stream
				def toggle_binary(widget):
					if widget.get_active():
						self.save_area.document.stream = key
					else:
						try:
							import gpg
							self.save_area.document.stream = gpg.MyGnuPG().get_armored(key)
						except:
							rox.alert(_('Conversion from binary to text failed!'))
							self.delete_choice.set_active(True)		 		
				self.vbox.add(self.save_area)
				self.delete_choice = g.CheckButton(_('Binary'))
				self.delete_choice.connect('toggled', toggle_binary)
				self.delete_choice.set_active(True)
				self.vbox.pack_start(self.delete_choice, True, True, 0)
				#self.delete_choice.show()
				
		def save_key(widget):
			if self.savebox:
				self.savebox.destroy()
			self.mime_type = 'application/pgp-keys'
			self.stream = self.key.value
			name = self.key.keyID+'.gpg'
			if roxUI():		
				self.savebox = SaveKeyBox(self, name, self.mime_type)
				self.savebox.show()
			else:
				self.FileChooser(os.environ.get('HOME'), name, self.mime_type)
		
		keybuttons = g.HBox(False)
		button_import = g.Button(_('Import Key'))
		button_save = g.Button(_('Save Key'))
		button_save.set_sensitive(False)
		button_import.set_sensitive(False)
		button_import.connect('clicked', import_key)
		button_save.connect('clicked', save_key)
		keybuttons.pack_start(button_save, True, False, 0)
		keybuttons.pack_start(button_import, True, False, 0)
		dialog.vbox.pack_start(keybuttons, True, True, 0)
		
		if iscommand('gpg'):
			import gpg
			keys = gpg.MyGnuPG().get_public_keys()
			newkey = self.key
			if newkey:
				if newkey.keyID in keys.keys():
					newkey = None
				else:
					keys[newkey.keyID] = newkey.UserID
					button_import.set_sensitive(True)
				active = keys.keys().index(self.key.keyID)
				button_save.set_sensitive(True)
			else:
				active = -1	
				
			gpg_keys = keys.keys()
			
			for UserID in keys.values(): combo.append_text(UserID)
			combo.set_active(active)
		else:
			info = g.Label(_('No PGP support - please install GnuPG!'))
			info.set_alignment(0, 1)
			info.set_justify(g.JUSTIFY_LEFT)
			info.set_line_wrap(True)
			hbox = g.HBox(False, 4)
			image = g.Image()
			image.set_from_stock(g.STOCK_DIALOG_INFO, g.ICON_SIZE_BUTTON)
			align = g.Alignment(0, 0, 0, 0)
			align.add(image)
			hbox.pack_start(align, False, True, 0)
			hbox.pack_start(info, False, True, 0)
			spacer = g.EventBox()
			spacer.set_size_request(6, 6)
			dialog.vbox.add(hbox)
			dialog.vbox.add(spacer)
			
		# CLASS setting
		widgets.section_header(dialog.vbox, _('Classification'))
		combo_class = g.combo_box_new_text()
		class_entry = combo_class.child		
		classes_i18n  = [_('Public'), _('Private'), _('Confidential')]
		classes_vcard = ['PUBLIC', 'PRIVATE', 'CONFIDENTIAL']
		for c in classes_i18n:
			combo_class.append_text(c)			
		if self.classification not in classes_vcard:
			classes_vcard.append(self.classification)
			classes_i18n.append(self.classification)
		combo_class.set_active(classes_vcard.index(self.classification))				
		dialog.vbox.pack_start(combo_class, True, True, 0)
						
		def got_response(widget, response):
			if response == g.RESPONSE_OK:		
				self.classification = classes_vcard[combo_class.get_active()]
			elif response == g.RESPONSE_NO:
				self.key_button.destroy()
				self.key = None
			self.update_discard()
			dialog.destroy()
		
		rox.toplevel_ref()
		dialog.connect('response', got_response)
		dialog.connect('destroy', lambda w: rox.toplevel_unref())
		dialog.show_all()	
			
	def edit_image(self, image_property, button, title):
		"""Image property dialog"""
		dialog = g.Dialog(title, self, g.DIALOG_DESTROY_WITH_PARENT, \
										(g.STOCK_DELETE, g.RESPONSE_NO,
										g.STOCK_SAVE,g.RESPONSE_YES,
										g.STOCK_OK, g.RESPONSE_OK))
		dialog.set_default_response(g.RESPONSE_OK)
		dialog.vbox.set_spacing(5)
		image = g.Image()
		image.set_from_pixbuf(image_property.pixbuf)
		dialog.vbox.pack_start(image, True, True, 0)
		dialog.show_all()
		
		response = dialog.run()

		dialog.destroy()

		if response == g.RESPONSE_NO:
			button.destroy()
			gc.collect()
			return True
		elif response == g.RESPONSE_YES:
			if self.savebox:
				self.savebox.destroy()
			self.mime_type = str(image_property.mime_type)
			name = self.fileas+ ' - '
			self.stream = image_property.data
			if self.logo == image_property:
				name += _('Logo')
			else:
				name += _('Photo')		
			if roxUI():		
				self.savebox = SaveBox(self, name, self.mime_type)
				self.savebox.show()
			else:
				self.FileChooser(os.environ.get('HOME'), name, self.mime_type)
		else:
			return False
			
	def edit_sound(self, widget = None):
		"""Sound editing dialog"""
		dialog = g.Dialog(_("Contact Sound Bite"), self, g.DIALOG_DESTROY_WITH_PARENT, \
														(g.STOCK_DELETE, g.RESPONSE_NO, 
														g.STOCK_SAVE,g.RESPONSE_YES,
														g.STOCK_OK, g.RESPONSE_OK))
		dialog.set_default_response(g.RESPONSE_OK)
		dialog.vbox.set_spacing(5)
		#image = g.Image()
		#image.set_from_pixbuf(image_property.pixbuf)
		info = _("File format")+": "+str(self.audioformat)
		dialog.vbox.pack_start(g.Label(info), True, True, 0)
		dialog.show_all()
		
		response = dialog.run()

		dialog.destroy()

		if response == g.RESPONSE_NO:
			self.sound_button.destroy()
			self.sound = None
			gc.collect()
			self.update_discard()
		elif response == g.RESPONSE_YES:
			if self.savebox:
				self.savebox.destroy()
			self.mime_type = str(self.audioformat)
			self.stream = self.sound
			name = self.fileas+' - '+_('Sound')
			if roxUI():		
				self.savebox = SaveBox(self, name, self.mime_type)
				self.savebox.show()
			else:
				self.FileChooser(os.environ.get('HOME'), name, self.mime_type)
		
	def edit_photo(self, widget = None):
		"""Photo editing dialog"""
		if self.edit_image(self.photo, self.photo_button, _('Contact Photo')):
			del self.photo
			self.photo = None
			self.update_discard()

	def edit_logo(self, widget = None):
		"""Logo editing dialog"""
		if self.edit_image(self.logo, self.logo_button, _('Contact Logo')):
			del self.logo
			self.logo = None
			self.update_discard()

	def edit_note(self, widget = None):
		"""Note taking dialog"""
		dialog = g.Dialog(_('Take Contact related Note'), self, g.DIALOG_DESTROY_WITH_PARENT, \
				(g.STOCK_CANCEL, g.RESPONSE_CANCEL, g.STOCK_OK, g.RESPONSE_OK))
		dialog.set_default_response(g.RESPONSE_OK)
		dialog.vbox.set_spacing(5)
		buffer = g.TextBuffer()
		buffer.set_text(self.note_text)
		note_entry = g.TextView(buffer)
		note_entry.set_wrap_mode(g.WRAP_WORD)
		
		note_entry.set_size_request(g.gdk.screen_width() / 4, g.gdk.screen_height() / 4)
		note_entry.modify_base(g.STATE_NORMAL, g.gdk.color_parse('#fdcd12'))
		#note_entry.set_width_chars(70)
		#note_entry.set_activates_default(True)
		dialog.vbox.pack_start(note_entry, True, True, 0)
	
		def got_response(widget, response):
			if response == g.RESPONSE_OK:
				self.note_text = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter())
			dialog.destroy()
			
			if self.note_text == '':
				if self.__dict__.has_key('note_button'):
					self.note_button.destroy()
			else:
				self.add_notebutton()	
		
		rox.toplevel_ref()
		dialog.connect('response', got_response)
		dialog.connect('destroy', lambda w: rox.toplevel_unref())
		dialog.show_all()


	def update_discard(self, widget = None):
		"Shade/unshade the Save button."
		s = self.uimanager.get_widget("/Toolbar/Save")
		s.set_sensitive(self.changed(widget))
	
	def changed(self, widget = None):
		"""Check whether any entries have different values (whether there are changes to save)."""
		if self.revert == None or self.revert == {}:
			return False
		if widget == self.entry.name:
			self.split_name()
			self.update_title()

		for entry in self.entry.__dict__.values():
			if entry.get_text() != self.revert[entry]:
				return True
		
		self.bday.string = self.bday_to_string()
		if self.bday.string != self.revert_bday:
			return True

		if self.photo:
			if	not self.revert_photo or \
	  			self.photo.b64 != self.revert_photo.b64 or \
	  			self.photo.uri != self.revert_photo.uri:
					return True
		elif self.revert_photo:
			return True

		if self.logo:
			if	not self.revert_logo or \
	  			self.logo.b64 != self.revert_logo.b64 or \
	  			self.logo.uri != self.revert_logo.uri:
					return True			
		elif self.revert_logo:
			return True

		if self.sound != self.revert_sound:
			return True
			
		if self.key != self.revert_key:
			return True
			
		if self.classification != self.revert_classification:
			return True
			
		if self.tz != self.revert_tz:
			return True
				
		if self.geo !=self.revert_geo:
			return True
				
		return False

	def name_change(self, widget = None):
		self.split_name()
		self.update_title()
	
	def match(self, completion, model, iter):
		name =  model[iter][0]
		#print name, 'was selected'
		search_folder = self.card_folder.value
		filename = os.path.join(search_folder,self.match_dict[name])
		if not os.path.exists(filename):
			rox.alert(_("Matching file\n%s\nno longer exists.") % filename)
		else:
			self.load(filename)
			self.init_discard()
					
	def search(self, widget = None):

		from datetime import date
		today = date.today()
		(year, month, day) = (today.year, today.month, today.day)
		today = today.toordinal() # for rapid date deltas in days
	
		self.liststore.clear()
		self.bday_dict = {}
		self.match_dict = {}
		
		search_folder = self.card_folder.value
		if not os.path.exists(search_folder):
			if not self.mkfolder(search_folder):
				return

		stack = os.listdir(search_folder)
		for card in stack:
			if mime.get_type(card, 1) != self.mime_type_vcard:
				continue
			self.get_dict_from_file(os.path.join(search_folder, card))
			if self.dict.has_key("N"):
				namelist = string.split(self.dict["N"], ';')
				if len(namelist)>1:
					name = namelist[1]+' '+namelist[0]
					self.liststore.append([name])
					self.match_dict[name] = card
					name = namelist[0]+', '+namelist[1]
					self.liststore.append([name])
					self.match_dict[name] = card
				elif len(namelist)>0:
					self.liststore.append([namelist[0]])
					self.match_dict[namelist[0]] = card
			elif self.dict.has_key("FN"):
				namelist = string.split(self.dict["FN"], 1)
				if len(namelist)>1:
					name = namelist[0]+' '+namelist[1]
					self.liststore.append([name])
					self.match_dict[name] = card
					name = namelist[1]+', '+namelist[0]
					self.liststore.append([name])
					self.match_dict[name] = card
				elif len(namelist)>0:
					self.liststore.append([namelist[0]])
					self.match_dict[namelist[0]] = card

			if self.dict.has_key("BDAY"):
				(y, m, d) = self.bday_from_dict(self.dict["BDAY"])
				if (month > m) or (month >= m and day > d):
					datum = date(year+1, m, d) # birthday behind for this year, check for next year
				else:
					datum = date(year, m, d) # birthday yet to come for this year
				datum = datum.toordinal()
				self.bday_dict[card] = datum

		
		min = 9999999
		for datum in self.bday_dict.values():
			if datum < min: min = datum
		self.closest_bday = min
		min_days = min - today

		if min != 9999999:
			if min_days == 0:
				id = "rox-contacts-cake-red"
				tip = _("Someone's birthday today")
			elif min_days == 1:
				id = "rox-contacts-cake-yellow"
				tip = _("Someone's birthday tomorrow")
			elif min_days <= 7:
				id = "rox-contacts-cake-green"
				tip = _("Next birthday in %s days") % min_days
			else:
				id = "rox-contacts-cake-white"
				min_date = date.fromordinal(min)
				tip = _("Next birthday in %s days") % min_days + ' (' + min_date.strftime("%x") + ')'
			self.cake_button = g.ToolButton(id)
			self.toolbar.insert(self.cake_button, -1)
			self.cake_button.set_tooltip(g.Tooltips(), tip)
			self.cake_button.show()
		#print self.bday_alert		
		#print min_days
		#print min
		#print date.fromordinal(min)
		#print self.bday_dict
		#print self.match_dict

	def escape(self, s):
		"""escape vcard value string"""
		s = s.replace(',', '\,')
		s = s.replace(';', '\;')
		s = s.replace('\n', '\\n')
		s = s.replace('\r', '\\r')
		s = s.replace('\t', '\\t')
		return s
	
	def unescape(self, s):
		"""unescape vcard value string"""
		s = s.replace('\,', ',')
		s = s.replace('\;', ';')
		s = s.replace('\\n', '\n')
		s = s.replace('\\r', '\r')
		s = s.replace('\\t', '\t')
		return s

	def split_name(self):
		"""Split name heuristically into vcard name components"""
		self.contactname.clear()
		name =  self.entry.name.get_text()
		if name == '' or name.isspace(): return
		components = name.split()
		pos = 0
		unclassified = []
		classified = []
		i = -1
		for item in components:
			i += 1 ; classified.append(0)
			if item in self.learned_names.prefixes and pos <=1:
				pos = 1 ; self.contactname.prefixes.append(item)
				classified[i] = 1
				continue
			#print self.learned_names.given
			if item in self.learned_names.given and pos <=2:
				#print "given"
				pos = 2 ; self.contactname.given.append(item)
				classified[i] = 2
				continue
			if item in self.learned_names.additional and pos <=3:
				pos = 3 ; self.contactname.additional.append(item)
				classified[i] = 3
				continue
			if item in self.learned_names.family and pos <=4:
				#print "family"
				pos = 4 ; self.contactname.family.append(item)
				classified[i] = 4
				continue
			if (item in self.learned_names.suffixes and pos <=5) or (pos == 5):
				pos = 5 ; self.contactname.suffixes.append(item)
				classified[i] = 5
				continue
			unclassified.append(i)

		for i in unclassified:
			if len(components) == 1:
				self.contactname.family.append(components[i])
				break
			elif len(components) == 2:
				if len(self.contactname.given) == 0:
					self.contactname.given.append(components[i])
					continue
				self.contactname.family.append(components[i])
			else:
				if len(self.contactname.given) == 0:
					self.contactname.given.append(components[i])
					continue
				if len(self.contactname.additional) == 0:
					self.contactname.additional.append(components[i])
					continue
				#if components[i]:
				self.contactname.family.append(components[i])			
			

	def fullname_from_contactname(self):
		def get_names(list):
			names = ''
			for item in list: names += ' '+item
			return names	
		fn  = get_names(self.contactname.prefixes)
		fn += get_names(self.contactname.given)
		fn += get_names(self.contactname.additional)
		fn += get_names(self.contactname.family)
		fn += get_names(self.contactname.suffixes)
		return self.escape(fn.lstrip())

	def name_from_contactname(self):
		def get_names(list):
			names = ''
			for item in list: names += ' '+item
			return self.escape(names.lstrip())
		n  = get_names(self.contactname.family)+';'
		n += get_names(self.contactname.given)+';'
		n += get_names(self.contactname.additional)+';'
		n += get_names(self.contactname.prefixes)+';'
		n += get_names(self.contactname.suffixes)
		return n

	def bday_to_string(self):
		if self.bday.checked.get_active():
			return	self.bday.year.get_text() + '-' + \
					self.bday.month.get_text() + '-' + \
					self.bday.day.get_text()
		else:
			return ''
			

	def bday_from_dict(self, value):
		"""return (yyyy/mm/dd) from card dictionary"""
		try:
			(y, m, d) = string.split(value, "-", 2)
			d = d.split('T')
			if y.isdigit() and m.isdigit() and d[0].isdigit():
				year = int(y); month = int(m); day = int(d[0])
				if month >=1 and month <=12:
					if day >= 1 and day <=31:
						return (year, month, day)
		except:
			pass
		return (None, None, None)

	def create_filename(self):
		def get_name(list): 
			if len(list) == 0: return ''
			else: return list[0]
		file_as = self.file_as.value
		if file_as == 'simple': 
			return get_name(self.contactname.given)+' '+\
				   get_name(self.contactname.family)
		elif file_as == 'full': 
			return self.fullname_from_contactname()
		elif file_as == 'reverse_comma':
			return get_name(self.contactname.family)+', '+\
				   get_name(self.contactname.given)
		elif file_as == 'reverse':
			return get_name(self.contactname.family)+' '+\
				   get_name(self.contactname.given)
		return ''

	def set(self, widget = None):
		self.card = []
		self.split_name()

		def get_simple_type(widget, type):
			val = self.escape(widget.get_text())
			if val != '':
				self.card += [ type + ':' + val ]
		
		# N
		self.card += [ "N:" + self.name_from_contactname() ]
		
		# FN			
		self.card += [ "FN:" + self.fullname_from_contactname() ]

		# NICKNAME
		get_simple_type(self.entry.nick, "NICKNAME")
		
		# BDAY
		if self.bday.checked.get_active():
			self.card += [ "BDAY:" + self.bday_to_string() ]

		# ADR
		adr =   self.entry.pobox.get_text() + ';;' + \
				self.entry.street.get_text() + ';' + \
				self.entry.city.get_text() + ';' + \
				self.entry.state.get_text() + ';' + \
				self.entry.zip.get_text() + ';' + \
				self.entry.country.get_text()
		if string.strip(adr, ';') != '':
			self.card += [ "ADR;TYPE=HOME,POSTAL,PARCEL:" + adr ]

		adr =	self.entry.work_pobox.get_text() + ';;' + \
				self.entry.work_street.get_text() + ';' + \
				self.entry.work_city.get_text() + ';' + \
				self.entry.work_state.get_text() + ';' + \
				self.entry.work_zip.get_text() + ';' + \
				self.entry.work_country.get_text()
		if string.strip(adr, ';') != '':
			self.card += [ "ADR;TYPE=WORK,POSTAL,PARCEL:" + adr ]             

		# CATEGORIES
		# don't call get_simple_type() to avoid escaping of commas
		val = self.entry.categories.get_text()
		if val != '':
			self.card += [ 'CATEGORIES:' + val ]
				
		# TEL
		def get_home_tel(entry, option):
			number = entry.get_text()
			if number != '':
				self.card += [ 'TEL;TYPE=HOME,'+tel_types[option.get_active()] + ':' + number ]
			
		get_home_tel(self.entry.tel_1, self.type_tel_1)
		get_home_tel(self.entry.tel_2, self.type_tel_2)
		get_home_tel(self.entry.tel_3, self.type_tel_3)
		get_home_tel(self.entry.tel_4, self.type_tel_4)
		
		def get_work_tel(entry, option):
			number = entry.get_text()
			if number != '':
				self.card += [ 'TEL;TYPE=WORK,'+tel_types[option.get_active()] + ':' + number ]

		get_work_tel(self.entry.work_tel_1, self.type_work_tel_1)
		get_work_tel(self.entry.work_tel_2, self.type_work_tel_2)
		get_work_tel(self.entry.work_tel_3, self.type_work_tel_3)
		get_work_tel(self.entry.work_tel_4, self.type_work_tel_4)

		# IM
		def get_im(entry, option):
			url = entry.get_text()
			if url != '':
				self.card += [im_types[option.get_active()] + ';TYPE=HOME:' + url]

		get_im(self.entry.im_1, self.type_im_1)
		get_im(self.entry.im_2, self.type_im_2)
		get_im(self.entry.im_3, self.type_im_3)
		get_im(self.entry.im_4, self.type_im_4)
		
		def get_work_im(entry, option):
			url = entry.get_text()
			if url != '':
				self.card += [im_types[option.get_active()] + ';TYPE=WORK:' + url]

		get_work_im(self.entry.work_im_1, self.type_work_im_1)
		get_work_im(self.entry.work_im_2, self.type_work_im_2)
		get_work_im(self.entry.work_im_3, self.type_work_im_3)
		get_work_im(self.entry.work_im_4, self.type_work_im_4)
				
		# ORG
		get_simple_type(self.entry.org, 'ORG')           

		# EMAIL
		get_simple_type(self.entry.email_1, "EMAIL;TYPE=INTERNET,HOME,PREF")
		get_simple_type(self.entry.email_2, "EMAIL;TYPE=INTERNET,HOME")
		get_simple_type(self.entry.work_email, "EMAIL;TYPE=INTERNET,WORK")

		# URL
		get_simple_type(self.entry.url, "URL;TYPE=INTERNET,HOME")
		get_simple_type(self.entry.work_url, "URL;TYPE=INTERNET,WORK")

		# TITLE
		get_simple_type(self.entry.jobtitle, "TITLE")

		# ROLE
		get_simple_type(self.entry.role, "ROLE")
		
		# CLASSIFICATION
		self.card += [ "CLASS:" + self.classification ]

		# NOTE
		val = self.escape(self.note_text)
		if val != '' and not val.isspace():
			self.card += [ 'NOTE:' + val ]

		# UID
		if self.uuid == '':
			import uuid
			self.uuid = uuid.uuid()
			# uuidgen from e2fsprogs is superior, but Linux only,
			# we use a python only uuid generator, avoiding deployment headache
			#pipe = os.popen('uuidgen', 'r')
			#if pipe:
				#self.uuid = pipe.readline().strip("\r\n")
				#pipe.close()
		self.card += [ "UID:" + self.uuid ]
		
		# TZ
		if self.tz:
			import time
			self.card += [ "TZ;VALUE=text:%s;%s;%s" % (self.tz[0], self.tz[1], self.tz[2]) ]

		# GEO
		if self.geo:
			self.card += [ "GEO:%f;%f" % (self.geo[0], self.geo[1]) ]
			
		# KEY	
		if self.key:
			key = base64.encodestring(self.key.value).replace('\n', '')
			self.card += [ "KEY;ENCODING=b:" + key ]
						
		if self.sound: 
			sound = base64.encodestring(self.sound).replace('\n', '')
			self.card += [ "SOUND;ENCODING=b;TYPE=%s:" % str(self.audioformat) + sound ]
		if self.photo: 
			photo = self.photo.b64.replace('\n', '')
			self.card += [ "PHOTO;ENCODING=b;TYPE=%s:" % self.photo.mime_type + photo ]
		if self.logo:  
			logo = self.logo.b64.replace('\n', '')
			self.card += [ "LOGO;ENCODING=b;TYPE=%s:"  % self.logo.mime_type + logo  ]

		from datetime import datetime
		self.rev = datetime.utcnow().isoformat().split('.', 1)[0]+'Z'
		self.card += [ "REV:"+ self.rev ]

		# we don't want custom properties to get lost upon saving.
		# rationale: Contacts can be used to edit the canonical
		# properties without the user having to fear data loss
		# with vCards from others sources.
		for unknown in self.unknown_properties:
			self.card += [ unknown+':'+self.unknown_properties[unknown] ]

	def save_to_stream(self, stream):
		print >>stream, self.stream
		#try:
			#import xattr
			#xattr.setxattr(stream, "user.mime_type", MIME_TYPE_VCARD)
			#xattr.setxattr(stream, "user.creator", "Contacts")
		#except:
			#pass
			
	def timeout(self, pid):
		pidp, r = os.waitpid(pid, os.WNOHANG)
		return False
		
	def command(self, argv):
		pid = os.spawnvp(os.P_NOWAIT, argv[0], argv)
		pidp, r = os.waitpid(pid, os.WNOHANG)
		if pidp == 0:
			gobject.timeout_add(10000, self.timeout, pid)
			
	def command2(self, argv):
		#from rox import processes
		#rox.processes.PipeThroughCommand(argv, None, file('output', 'w')).wait()
		#rox.processes.PipeThroughCommand(argv, None, None).wait()
		os.execvp(argv)
		
	def setfattr(self, uri):
		if sys.platform == 'linux2':
			if iscommand('setfattr'):
				self.command(['setfattr', '-n', 'user.mime_type', '-v', self.mime_type, '-h' , uri])
				self.command(['setfattr', '-n', 'user.creator', '-v', 'Contacts', '-h' , uri])
		elif sys.platform == 'sunos5':
			if iscommand('runat'):
				self.command(['runat', uri, 'echo', self.mime_type, '>user.mime_type' ])
				self.command(['runat', uri, 'echo', 'Contacts', '>user.creator'])
			
	def set_uri(self, uri):
		self.setfattr(uri)
		self.init_discard()
		self.update_title()

	def vcard_from_window(self):
		"""Turn instance window into vcard string"""
		"""return value: full path for writing"""
		name = self.entry.name.get_text()
		self.split_name()
		
		if name == '' or name.isspace():
			name = self.entry.org.get_text()		
		if name == '' or name.isspace():
			rox.info(_("Please enter a name or organization first, to identify the contact!"))
			return None
		
		try: # we already have a filename
			folder = os.path.dirname(self.uri)
		except:
			folder = self.card_folder.value
			if not os.path.exists(folder):
				if not self.mkfolder(folder):
					return None

		if self.fileas != None:
			name = self.fileas
		# make sure to use unique filenames for contacts
		# the uid tells us if it's really the same contact,
		# which we can simply overwrite
		stack = os.listdir(folder)
		filename = name+'.vcf'
		i = 1
		while filename in stack:
			self.get_dict_from_file(os.path.join(folder, filename))
			if not self.dict.has_key('UID') or self.uuid == self.dict['UID']:
				break
			else:
				i += 1 ; filename = name + ' '+str(i)+'.vcf'
						
		self.set()
		self.set_title(name)

		self.vcard = "BEGIN:VCARD\nVERSION:3.0\n"

		def fold(line):
			"""RFC 2425 folding procedure"""
			#splits = len(line) / 74
			l = string.split(line, ':', 1)
			if len(l) == 1:
				return line # fishy line
			key = l[0] ; value = l[1]
			 
			fline = key+':'
			pos = 75-len(fline)
			fline += value[0:pos]

			while len(value)-pos >= 76:
				start = pos ; pos += 75
				fline += foldmark+value[start:pos]
				#print "start:"
				#print value[start:pos]
				#print "end:"
			if pos < len(value):
				fline += foldmark+value[pos:]
				
			return fline

		#print self.card
						
		for line in self.card:
			if len(line) > 75:
				self.vcard += fold(line)
			else:
				self.vcard += line
			self.vcard += '\n'
			
		self.vcard += "PRODID:Contacts %s\n" % self.version()
		self.vcard += "END:VCARD"

		return os.path.join(folder, filename)

	def mkfolder(self, folder):
		try:
			os.mkdir(folder)
		except:
			rox.alert(_("Could not create folder %s.\n") % folder)
			return False
		return True	
		
	def FileChooser(self, path, name, mime_type, discard = 0):
		chooser = g.FileChooserDialog(_("Save As"), self, \
							g.FILE_CHOOSER_ACTION_SAVE, \
							(g.STOCK_CANCEL, g.RESPONSE_CANCEL, \
							g.STOCK_OK,g.RESPONSE_OK))		
		if discard:
			def discard_clicked(event):
				chooser.destroy()
				self.discard()
			button = rox.ButtonMixed(g.STOCK_DELETE, _('_Discard'))
			button.connect('clicked', discard_clicked)
			button.unset_flags(g.CAN_FOCUS)
			button.set_flags(g.CAN_DEFAULT)
			chooser.action_area.pack_end(button, False, True, 0)
			chooser.action_area.reorder_child(button, 0)
			button.show_all()		
		chooser.set_current_folder(path)
		chooser.set_current_name(name)
		chooser.set_default_response(g.RESPONSE_OK)
		filter = g.FileFilter()
		filter.set_name(mime.lookup(mime_type).get_comment())
		filter.add_mime_type(mime_type)
		chooser.add_filter(filter)
		response = chooser.run()		
		if response == g.RESPONSE_OK:
			self.save_to_uri(chooser.get_filename())
		chooser.destroy()
	
	def save_to_uri(self, uri):
		try:		
			self.save_to_file(uri)
			self.set_uri(uri)
		except:
			rox.alert(_("Couldn't create or change file %s" % uri))
					
	def save(self, widget = None):
		"""Save instance window as vcard to disk"""
		uri = self.vcard_from_window()
		if uri: 
			self.stream = self.vcard
			self.mime_type = MIME_TYPE_VCARD
			self.save_to_uri(uri)		
		
	def saveas(self, widget = None, discard = 0):
		"""Open SaveBox for writing instance window as vcard to disk"""
		uri = self.vcard_from_window()
		if uri:
			self.mime_type = MIME_TYPE_VCARD
			self.stream = self.vcard
			if roxUI():			
				if self.savebox: self.savebox.destroy() 
				self.savebox = SaveBox(self, uri, MIME_TYPE_VCARD, discard)
				self.savebox.show()
			else:
				self.FileChooser(self.card_folder.value, self.fileas+'.vcf', MIME_TYPE_VCARD, discard)
			
				
	def noname(self):		
		name = self.entry.name.get_text()
		if name == '' or name.isspace():
			name = self.entry.org.get_text()
		return name == '' or name.isspace()		
		
	def discard(self):
		self.destroy()
		
	def close(self, button = None):
		if not self.noname() and self.changed():
			self.saveas(discard = 1)
		else:
			self.destroy()
                        
	def delete_event(self, window, event):
		if not self.noname() and self.changed():
			self.saveas(discard = 1)
			return 1
		return 0

	def destroyed(self, widget):
		rox.app_options.remove_notify(self.options_changed)

	def options_changed(self):
		self.update_title()
	
	def update_title(self):
		"""Set window title to vcard filename or default title"""
		if len(self.contactname.family) > 0:
			title = self.create_filename()
			self.fileas = title
		else:
			title = self.entry.org.get_text()
			if title == '' or title.isspace():
				title = TITLE_DEFAULT
		if self.rev:
			title += ' ('+_('Rev')+': '+self.rev +')'
		self.set_title(title)

									
	def get_dict_from_file(self, filename):
		"""Turn each line of a vcard into a dictionary item."""
		"""Valid vcard lines are colon separated key/value pairs."""

		def to_utf8(value, key):
			"""convert value from any to utf-8"""
			
			encoding = self.encoding

			# deal with vCard v2.1 inline encodings
			keylist = key.split(';')
			for item in keylist:
				if item.find("QUOTED-PRINTABLE") >= 0:
					#for n in keylist:
						#if n == "NOTE":
							#print "********before************"
							#print value
							#print "********after************"
							#decoder = codecs.getdecoder('quopri_codec')
							#print decoder(value)[0]
					value = quopri.decodestring(value)	 
					#print value
				if item.find("CHARSET") == 0:
					encoding = item.split('=', 1)[0].lower()
			
			# convert from detected encoding to utf-8
			try:
				decoder = codecs.getdecoder(encoding)
				value = decoder(value)[0]
				#print value
			except:
				pass
						
			return value
			
		self.dict = {}
		self.get_card_from_file(filename)
		for l in self.card:
			line = string.strip(l, "\r\n")
			try:
				(key, value) = string.split(line, ":", 1)
			except:
				continue
			key = string.upper(key)
			
			# avoid loss of dupes
			if self.dict.has_key(key):
				if key.find('EMAIL')==0 or key.find('TEL')==0 or key.find('ADR')==0:
					if key.find(',PREF')>=0:
						self.dict[key.replace(',PREF','')] = self.dict[key]
					elif key.find(':PREF')>=0:
						self.dict[key.replace('PREF','', 1)] = self.dict[key]
					else:
						self.dict[key+',PREF'] = self.dict[key]
			
			self.dict[key] = to_utf8(value, key)

		#print self.dict
			
	def get_entries_from_dict(self):
		"""Fill window entries from vcard dictionary"""
		fullname = '' ; namelist = None
		# store unhandled properties here, 
		# so they don't get lost when saving
		self.unknown_properties = {}
		
		def set_im(type, value, valuelist):
			value = self.unescape(value)
			history = im_types.index(type)
			if "WORK" in valuelist:
				if self.entry.work_im_1.get_text() == '':
					self.entry.work_im_1.set_text(value)
					self.type_work_im_1.set_active(history)
				elif self.entry.work_im_2.get_text() == '':
					self.entry.work_im_2.set_text(value)
					self.type_work_im_2.set_active(history)
				elif self.entry.work_im_3.get_text() == '':
					self.entry.work_im_3.set_text(value)
					self.type_work_im_3.set_active(history)
				elif self.entry.work_im_4.get_text() == '':
					self.entry.work_im_4.set_text(value)
					self.type_work_im_4.set_active(history)     
			else:
				if self.entry.im_1.get_text() == '':
					self.entry.im_1.set_text(value)
					self.type_im_1.set_active(history)
				elif self.entry.im_2.get_text() == '':
					self.entry.im_2.set_text(value)
					self.type_im_2.set_active(history)
				elif self.entry.im_3.get_text() == '':
					self.entry.im_3.set_text(value)
					self.type_im_3.set_active(history)
				elif self.entry.im_4.get_text() == '':
					self.entry.im_4.set_text(value)
					self.type_im_4.set_active(history)

		def get_image_property(valuelist, value):
			property = ImageProperty()
			if "VALUE=URI" in valuelist:
				try:			
					property.from_uri(value)
				except:
					property.uri = value
			else:
				try:
					"ENCODING=B" in valuelist
					property.from_base64(value)
				except:
					del property
					property = None
			return property
		
		for key in self.dict:

			value = self.dict[key]

			types = []
			
			try:
				typestring = key.split("TYPE=", 1)[1]
				if typestring.find(","):
					types = typestring.split(",")
			except:
				pass

			#if types: print key ; print types
			
			key = key.replace("TYPE=", "")
			key = key.replace(";", ",")
			try:
				valuelist = string.split(key, ",")
				type = valuelist[0]
				del valuelist[0]
				#print valuelist
			except:
				valuelist = []
				type = key
			
			if type == "BDAY":
				(y, m, d) = self.bday_from_dict(value)
				if d:
					self.bday.year.set_value(y)
					self.bday.month.set_value(m)
					self.bday.day.set_value(d)
					self.bday.checked.set_active(True)
			
			elif type == "URL":
				value = self.unescape(value)
				if value.find("callto://") >=0 or value.find("h323://") >=0:
					if "WORK" in valuelist:
						self.entry.work_videoconference.set_text(value)
					else:
						self.entry.videoconference.set_text(value)    
				elif "WORK" in valuelist:
					self.entry.work_url.set_text(value)
				else:
					self.entry.url.set_text(value)
			elif type == "EMAIL":
				value = self.unescape(value)
				if "WORK" in valuelist:
					self.entry.work_email.set_text(value)
				else:
					if self.entry.email_1.get_text() == '':
						self.entry.email_1.set_text(value)
					else:
						self.entry.email_2.set_text(value)
			elif type == "ADR":
				value = self.unescape(value)
				if value.count(';') == 5:
					value += ';'
				if value.count(';') == 6:
					(box, extended, street, city, state, zip, country) = \
							string.split(value,';')
					if "WORK" in valuelist:
						self.entry.work_pobox.set_text(box)
						self.entry.work_street.set_text(street)
						self.entry.work_city.set_text(city)
						self.entry.work_state.set_text(state)
						self.entry.work_zip.set_text(zip)
						self.entry.work_country.set_text(country)
					else:      
						self.entry.pobox.set_text(box)
						self.entry.street.set_text(street)
						self.entry.city.set_text(city)
						self.entry.state.set_text(state)
						self.entry.zip.set_text(zip)
						self.entry.country.set_text(country)
				else:
					print "ADR malformed"
			elif type == "TEL":
				history = 0
				for i in valuelist:
					if i in tel_types:
						history = tel_types.index(i)
						break
					
				if "WORK" in valuelist:
					if self.entry.work_tel_1.get_text() == '':
						self.entry.work_tel_1.set_text(value)
						self.type_work_tel_1.set_active(history)
					elif self.entry.work_tel_2.get_text() == '':
						self.entry.work_tel_2.set_text(value)
						self.type_work_tel_2.set_active(history)
					elif self.entry.work_tel_3.get_text() == '':
						self.entry.work_tel_3.set_text(value)
						self.type_work_tel_3.set_active(history)
					elif self.entry.work_tel_4.get_text() == '':
						self.entry.work_tel_4.set_text(value)
						self.type_work_tel_4.set_active(history)
				else:
					if self.entry.tel_1.get_text() == '':
						self.entry.tel_1.set_text(value)
						self.type_tel_1.set_active(history)
					elif self.entry.tel_2.get_text() == '':
						self.entry.tel_2.set_text(value)
						self.type_tel_2.set_active(history)
					elif self.entry.tel_3.get_text() == '':
						self.entry.tel_3.set_text(value)
						self.type_tel_3.set_active(history)
					elif self.entry.tel_4.get_text() == '':
						self.entry.tel_4.set_text(value)
						self.type_tel_4.set_active(history)

			elif type == "ORG":
				value = self.unescape(value)
				self.entry.org.set_text(value)
			elif type == "CATEGORIES":
				self.entry.categories.set_text(value)
			elif type == "N":
				namelist = string.split(value, ';')          
			elif type == "FN":
				fullname = self.unescape(value)
			elif type == "NICKNAME":
				value = self.unescape(value)
				self.entry.nick.set_text(value)
			elif type == "TITLE":
				value = self.unescape(value)
				self.entry.jobtitle.set_text(value)
			elif type == "ROLE":
				value = self.unescape(value)
				self.entry.role.set_text(value)
			elif type == "NOTE":
				self.note_text = self.unescape(value)
				self.add_notebutton()
			elif type == "TZ":
				import time
				
				zoneinfo = value.split(';')
				tz = [None, None, None]
				offset = None
				for info in zoneinfo:
					if info.find('/') >=0:
						tz[2] = info
						offset = utc_offset(info)
					elif info.isalpha():
						tz[1] = info
						offset = utc_offset(info)
					elif info.strip('-').strip(':').isdigit():
						tz[0] = int(info)
							
				if offset:
					if not tz[0]: tz[0] = offset
					if not tz[1]: tz[1] = time.tzname[0]
				
				self.tz = tz
				self.add_geobutton()	
			elif type == "GEO": 
				(latitude, longitude) = value.split(';' ,1)
				self.geo = [float(latitude), float(longitude)]
				self.add_geobutton()
			elif type == "KEY":
				if iscommand('gpg'):
					try:
						import gpg
						k = gpg.Key()
						k.value = base64.decodestring(value)
						gnupg = gpg.MyGnuPG()
						(k.keyID, k.UserID) = gnupg.get_ID(k.value)
						self.key = k
						self.add_keybutton()
					except:
						pass
				else:
					self.add_keybutton()
					self.unknown_properties[key] = value
			elif type == "SOUND":
				if "VALUE=URI" in valuelist:
					self.uri_sound = value	
				elif "ENCODING=B" in valuelist:
					try:
						self.sound = base64.decodestring(value)
						self.write_soundfile()
						self.add_soundbutton()
					except:
						self.sound = None
						self.unknown_properties[key] = value	
				else:
					self.unknown_properties[key] = value
			elif type == "PHOTO":
				try:
					self.photo = get_image_property(valuelist, value)
					self.add_photobutton()
				except:
					self.photo = None
					self.unknown_properties[key] = value
			elif type == "LOGO": 
				try:
					self.logo = get_image_property(valuelist, value)
					self.add_logobutton()
				except:
					self.logo = None
					self.unknown_properties[key] = value	
			# Instant Messaging
			elif type in im_types:
				set_im(type, value, valuelist)
			elif type == "X-EVOLUTION-BLOG-URL":
				value = self.unescape(value)
			elif type == "X-EVOLUTION-FILE-AS":
				self.fileas = self.unescape(value)
				self.set_title(self.fileas)
			elif type == "UID":
				self.uuid = value
			elif type == "CLASS":
				self.classification = value
			elif type == "REV":
				self.rev = value
			elif type not in ['BEGIN', 'VERSION', 'PRODID', 'END']:
					self.unknown_properties[key] = value 	
			
		# N is potentially more precise as FN, so it takes preference 
		if namelist != None:
			self.contactname.family = namelist[0].split()
			if len(namelist) > 1:
				self.contactname.given = namelist[1].split()
				if len(namelist) > 2:
					self.contactname.additional = namelist[2].split()
					if len(namelist) > 3:
						self.contactname.prefixes = namelist[3].split()
						if len(namelist) > 4:
							self.contactname.suffixes = namelist[4].split()
			name = self.fullname_from_contactname()
			if not name.isspace():
				fullname = name
				self.learn_names()

		self.entry.name.set_text(fullname)
		self.split_name()
		#print self.contactname.prefixes
		#print self.contactname.given
		#print self.contactname.additional
		#print self.contactname.family
		#print self.contactname.suffixes

		#print self.unknown_properties
		
	
	def write_soundfile(self, widget = None):
		self.soundfilename = basedir.save_config_path('Contacts') + '/soundfile'
		soundfile = open(self.soundfilename, 'w')
		soundfile.write(self.sound)
		soundfile.close()
		mime_type = mime.get_type(self.soundfilename, 1)
		try:
			mime_type.media == 'audio'
			self.audioformat = mime_type
		except:
			raise
			
	def tooltip_note(self, widget):
		self.note_button.set_tooltip(g.Tooltips(), self.note_text)
		
	def add_notebutton(self, widget = None):	
		if not self.__dict__.has_key("note_button"):
			self.note_button = g.ToolButton('rox-stock-notes')
			self.toolbar.insert(self.note_button, -1)	
			self.note_button.connect('clicked', self.edit_note)
			self.note_button.child.connect('enter', self.tooltip_note)
			self.note_button.show()
			
	def tooltip_geo(self, widget):
		try:
			import time
			os.environ['TZ'] = self.tz[2]
			time.tzset()
			tip = self.tz[2] + ' ' + _('Local Time') + ': ' + time.strftime('%X %p')
		except:
			tip = _("Geo Data")
		if self.geo:
			LAT = _('Latitude')+': %f' % self.geo[0]
			LONG =_('Longitude')+': %f' % self.geo[1]
			tip += '\n'+LAT+' '+LONG		
		self.geo_button.set_tooltip(g.Tooltips(), tip)
			
	def add_geobutton(self, widget=None):
		if not self.__dict__.has_key("geo_button"):
			self.geo_button = g.ToolButton('rox-stock-timezone')
			self.toolbar.insert(self.geo_button, -1)
			self.geo_button.connect('clicked', self.edit_geo)
			self.geo_button.child.connect('enter', self.tooltip_geo)
			self.geo_button.show()	
			
	def tooltip_key(self, widget):
		tip = _('PGP Public Key')
		try:
			tip += ' ID: '+self.key.keyID+' \n'+self.key.UserID
		except:
			pass		
		self.key_button.set_tooltip(g.Tooltips(), tip)
	
	def add_keybutton(self, widget=None):
		if not self.__dict__.has_key("key_button"):
			self.key_button = g.ToolButton('rox-contacts-blue-key')
			self.toolbar.insert(self.key_button, -1)
			self.key_button.connect('clicked', self.edit_privacy)
			self.key_button.child.connect('enter', self.tooltip_key)
			self.key_button.show()
								
	def add_soundbutton(self, widget=None):
		if not self.__dict__.has_key("sound_button"):
			self.sound_button = g.ToolButton('rox-stock-sound')
			self.toolbar.insert(self.sound_button, -1)
			self.sound_button.connect('clicked', self.edit_sound)
			self.sound_button.child.connect('enter', self.play_sound)
			self.sound_button.set_tooltip(g.Tooltips(), str(self.audioformat))
			self.sound_button.show()

	def add_photobutton(self, widget=None):
		if not self.__dict__.has_key("photo_button"):
			self.photo_button = g.ToolButton('rox-contacts-photo')
			self.toolbar.insert(self.photo_button, -1)
			self.photo_button.connect('clicked', self.edit_photo)
			self.photo_button.set_tooltip(g.Tooltips(), str(self.photo.mime_type))
			self.photo_button.show()

	def add_logobutton(self, widget=None):
		if not self.__dict__.has_key("logo_button"):
			self.logo_button = g.ToolButton('rox-contacts-logo')
			self.toolbar.insert(self.logo_button, -1)
			self.logo_button.connect('clicked', self.edit_logo)
			self.logo_button.set_tooltip(g.Tooltips(), str(self.logo.mime_type))
			self.logo_button.show()
		
	def play_sound(self, widget=None):
		try:
			self.write_soundfile()
			handler = self.audio_handler.value
			if handler != '':
				self.command([handler, self.soundfilename ])
			elif roxUI():
				#FIXME: (why does it not work): filer.spawn_rox(uri) 
				self.command([filemanager, self.soundfilename ])
		except:
			pass		

	def get_encoding(self, message):
		"Returns (encoding, errors), or raises Abort to cancel."
		print "Returns (encoding, errors), or raises Abort to cancel."
		box = g.MessageDialog(self, 0, g.MESSAGE_QUESTION, g.BUTTONS_CANCEL, message)
		#box.set_has_separator(False)

		frame = g.Frame()
		box.vbox.pack_start(frame, True, True)
		frame.set_border_width(6)

		hbox = g.HBox(False, 4)
		hbox.set_border_width(6)

		hbox.pack_start(g.Label(_('Encoding:')), False, True, 0)
		combo = g.Combo()
		combo.disable_activate()
		combo.entry.connect('activate', lambda w: box.activate_default())
		combo.set_popdown_strings(known_codecs)
		hbox.pack_start(combo, True, True, 0)
		ignore_errors = g.CheckButton(_('Ignore errors'))
		hbox.pack_start(ignore_errors, False, True)

		frame.add(hbox)
		
		box.vbox.show_all()
		box.add_button(g.STOCK_CONVERT, g.RESPONSE_YES)
		box.set_default_response(g.RESPONSE_YES)

		while 1:
			combo.entry.grab_focus()
			
			resp = box.run()
			if resp != g.RESPONSE_YES:
				box.destroy()
				raise Abort

			if ignore_errors.get_active():
				errors = 'replace'
			else:
				errors = 'strict'
			encoding = combo.entry.get_text()
			try:
				codecs.getdecoder(encoding)
				break
			except:
				rox.alert(_("Unknown encoding '%s'") % encoding)
			
		box.destroy()

		return encoding, errors

	def get_encoding_unstable(self, message):
		"Returns (encoding, errors), or raises Abort to cancel."
		print "Returns (encoding, errors), or raises Abort to cancel."
		box = g.MessageDialog(self, 0, g.MESSAGE_QUESTION, g.BUTTONS_CANCEL, message)

		frame = g.Frame()
		box.vbox.pack_start(frame, True, True)
		frame.set_border_width(6)

		hbox = g.HBox(False, 4)
		hbox.set_border_width(6)

		hbox.pack_start(g.Label(_('Encoding:')), False, True, 0)
		combo = g.combo_box_new_text()
		entry = combo.child
		for item in known_codecs:
			combo.append_text(item)
		combo.set_active(0)
		#combo.disable_activate()
		#entry.connect('activate', lambda w: box.activate_default())
		hbox.pack_start(combo, True, True, 0)
		ignore_errors = g.CheckButton(_('Ignore errors'))
		hbox.pack_start(ignore_errors, False, True)

		frame.add(hbox)
		
		box.vbox.show_all()
		box.add_button(g.STOCK_CONVERT, g.RESPONSE_YES)
		box.set_default_response(g.RESPONSE_YES)
		
		def got_response(widget, response):			
			if resp != g.RESPONSE_YES:
				box.destroy()

			if ignore_errors.get_active():
				errors = 'replace'
			else:
				errors = 'strict'
			encoding = entry.get_text()
			try:
				codecs.getdecoder(encoding)
			except:
				rox.alert(_("Unknown encoding '%s'") % encoding)
			
			box.destroy()

		rox.toplevel_ref()
		box.connect('response', got_response)
		box.connect('destroy', lambda w: rox.toplevel_unref())
		box.show_all()

		return encoding, errors

	

	def decode_v30(self, contents):
		"""convert v3.0 card to utf-8"""
		# vcard version 3.0: encoding is global,
		# provided with the MIME header.
		# When we don't have the header, assume utf-8
		# if that fails, pop up dialog for the user to select
		# the encoding
		errors = 'strict'
		encoding = 'utf-8'
		while 1:
			decoder = codecs.getdecoder(encoding)
			try:
				data = decoder(contents, errors)[0]
				if errors == 'strict':
					assert '\0' not in data
				else:
					if '\0' in data:
						data = data.replace('\0', '\\0')
				break
			except:
				pass

			encoding, errors = self.get_encoding(
					_("Data is not valid %s. Please select the file's encoding.") % encoding)
	
		return encoding 

	def get_card_from_file(self, filename):
		"""Get card from file as list with lines unfolded"""
		self.card = []
		self.encoding = 'utf-8'
		try:
			if filename == '-':
				contents = sys.stdin.read()
			else:
				file = open(filename, 'r')
				contents = file.read()
				file.close()
		
			for mark in foldmarks:
				contents = contents.replace(mark, '')

			lines = contents.splitlines()
			i = -1 ; length = len(lines) ; nesting = 0
			while i <= length:
				i += 1
				self.card.append(lines[i])
				# handle embedded cards and multiple card files
				if string.upper(lines[i]).find("BEGIN:") >=0:
					nesting += 1
				elif string.upper(lines[i]).find("END:") >=0:
					nesting -= 1
				if nesting <= 0:
					break
				
			#print self.card
			if not (string.upper(contents).find("QUOTED-PRINTABLE") >=0 \
				or string.upper(contents).find("CHARSET=")) >=0:
				self.encoding = self.decode_v30(contents)

		except: 
			print "pass"
			pass

	def edit_name(self, widget):
		"""Name editing dialog"""
		dialog = g.Dialog(_('Edit Contact Name'), self, g.DIALOG_DESTROY_WITH_PARENT, \
				(g.STOCK_CANCEL, g.RESPONSE_CANCEL, g.STOCK_OK, g.RESPONSE_OK))
		dialog.set_default_response(g.RESPONSE_OK)
		dialog.vbox.set_spacing(5)
		sg = g.SizeGroup(g.SIZE_GROUP_HORIZONTAL)
		combobox_prefixes = widgets.comboboxentry_with_label(dialog.vbox, sg, _('Honoric Prefixes'), 30)
		entry_prefixes = combobox_prefixes.child		
		entry_given = widgets.entry_with_label(dialog.vbox, sg, _('Given Name'), 30)
		entry_additional = widgets.entry_with_label(dialog.vbox, sg, _('Additional Names'), 30)
		entry_family = widgets.entry_with_label(dialog.vbox, sg, _('Family Name'))
		combobox_suffixes = widgets.comboboxentry_with_label(dialog.vbox, sg, _('Honoric Suffixes'), 30)
		entry_suffixes = combobox_suffixes.child

		

		#print self.contactname.prefixes
		#print self.contactname.given
		#print self.contactname.additional
		#print self.contactname.family
		#print self.contactname.suffixes
		
		self.split_name()
		
		#print self.contactname.prefixes
		#print self.contactname.given
		#print self.contactname.additional
		#print self.contactname.family
		#print self.contactname.suffixes
		
		for item in self.learned_names.prefixes: combobox_prefixes.append_text(item)
		for item in self.learned_names.suffixes: combobox_suffixes.append_text(item)
		
		def set_names(list, entry):
			s = ''
			for item in list: s += ' '+item
			entry.set_text(s.lstrip())
		set_names(self.contactname.prefixes, entry_prefixes)
		set_names(self.contactname.given, entry_given)
		set_names(self.contactname.additional, entry_additional)
		set_names(self.contactname.family, entry_family)
		set_names(self.contactname.suffixes, entry_suffixes)
					
		#combobox_formatted = widgets.comboboxentry_with_label(dialog.vbox, sg, _('Formatted Name'))
		#entry_formatted = combobox_formatted.child
		#formatted_names = [ 'Custom', 'Simple Name',
		#'Full Name', 'Reverse Name with Comma', 
		#'Reverse Name', 'Organisation'] 
		#for item in formatted_names: combobox_formatted.append_text(item)
		#radio: Parse name automatically
					
		def got_response(widget, response):
			if response == g.RESPONSE_OK:
				def get_names(entry):
					text = entry.get_text()
					if not text == '' or text.isspace():
						return text.split()
					return []
				self.contactname.prefixes	= get_names(entry_prefixes)	
				self.contactname.given		= get_names(entry_given)
				self.contactname.additional = get_names(entry_additional)
				self.contactname.family		= get_names(entry_family)
				self.contactname.suffixes	= get_names(entry_suffixes)
				self.entry.name.set_text(self.fullname_from_contactname())
				self.update_title()
				
				# remember the appropriate types of the name parts to be used in split_name()
				self.learn_names()
			dialog.destroy()
		
		rox.toplevel_ref()
		dialog.connect('response', got_response)
		dialog.connect('destroy', lambda w: rox.toplevel_unref())
		dialog.show_all()
		
	def learn_names(self):
		self.learned_names.given = self.contactname.given
		self.learned_names.family = self.contactname.family
		self.learned_names.additional = self.contactname.additional
		for item in self.contactname.prefixes:
			if item not in self.learned_names.prefixes:
				self.learned_names.prefixes.append(item)
		for item in self.contactname.suffixes:
			if item not in self.learned_names.suffixes:
				self.learned_names.suffixes.append(item)

	def version(self):
		info=os.path.join(rox.app_dir, 'AppInfo.xml')
		app_info=AppInfo.AppInfo(info)
		return app_info.getAbout('Version')[1]

	def clear(self, widget = None):
		"""Delete all entries"""
		for item in self.entry.__dict__.values(): 
			item.set_text('')
		self.fileas = None
		self.rev = None
		self.uuid = ''
		self.note_text = ''
		self.classification = 'PUBLIC'
		self.tz = None
		self.geo = None
		self.key = None
		self.sound = None
		self.photo = None
		self.logo = None
		
		if self.__dict__.has_key("note_button"):
			self.note_button.destroy()		
		if self.__dict__.has_key("geo_button"):
			self.geo_button.destroy()	
		if self.__dict__.has_key("key_button"):
			self.key_button.destroy()
		if self.__dict__.has_key("sound_button"):
			self.sound_button.destroy()
		if self.__dict__.has_key("photo_button"):
			self.photo_button.destroy()
		if self.__dict__.has_key("logo_button"):
			self.logo_button.destroy()		
			
		self.bday.checked.set_active(False)
		self.contactname.clear()
		self.learned_names.clear()
		self.learned_names.prefixes = EContact.NamePrefix
		self.learned_names.suffixes = EContact.NameSuffix
		self.init_discard()
		self.update_title()

	def delete(self, widget = None):
		"""Delete the open card from disk"""
		if self.uuid and self.fileas:
			file = os.path.join(self.card_folder.value, self.fileas)+'.vcf'
			if os.path.exists(file):
				try:
					os.remove(file)
					self.search()
					#self.liststore.remove(self.fileas)
					#if len(self.contactname.family[0])>0:
						#self.liststore.remove(self.contactname.family[0])
				except:
					rox.alert(_('The card file %s could not be deleted') % file)

	def load(self, filename):
		"""Load vcard into window"""
		self.clear()
		self.set_title(_("Loading")+' '+filename+'...')
		self.get_dict_from_file(filename)
		self.get_entries_from_dict()
		self.update_title()

	def xds_load_from_stream(self, name, mimetype, stream):
		mime_type = mime.get_type(name)
		if mime_type == self.mime_type_vcard:	
			self.load(name)
			self.init_discard()
		elif mime_type.media == 'audio':
			try:
				soundfile = open(name, 'r')
				self.sound = soundfile.read()
				soundfile.close()
				self.audioformat = mime_type		
				self.update_discard()
				self.add_soundbutton()
			except:
				rox.alert(_('Error reading file %s') % name)
		elif mime_type.media == 'image':
			try:
				if self.notebook.get_current_page() == 0:
					# drag image onto "Home" tab -> assume PHOTO
					photo = ImageProperty()
					photo.from_file(name)			
					if self.photo:
						del self.photo
						self.photo = None
						self.photo_button.destroy()
					self.photo = photo
					self.update_discard()
					self.add_photobutton()
				else:
					# drag image onto "Work" tab -> assume LOGO
					logo = ImageProperty()
					logo.from_file(name)
					if self.logo:
						del self.logo
						self.logo = None
						self.logo_button.destroy()
					self.logo = logo
					self.update_discard()
					self.add_logobutton()				
			except:
				rox.alert(_('Error reading image file %s') % name)
		else:
			rox.info(_("This doesn't look like a vCard!"))
				
	def show(self, widget = None):
		filer.open_dir(self.card_folder.value)

	def show_options(self, button=None):
		"""Options edit dialog"""
		rox.edit_options()

	def toggle_sensitivity(self, b):
		self.actiongroup.set_sensitive(b.get_active())
		return

	def toggle_visibility(self, b):
		self.actiongroup.set_visible(b.get_active())
		return

	def me(self, widget = None):
		if self.uuid == '':
			rox.info(_('You need to open a card I can assign to you!'))
			return
			
		self.uuid_me = self.uuid
		dir = basedir.save_config_path('Contacts')
		try:
			file = open ( dir + '/ME', 'w' )
			file.write (self.uuid)
			file.close()
		except:
			rox.alert(_('Can\'t save this option to disk!\n(Disk mounted read-only or insufficient permissions)'))

	def find_me(self, widget = None):
		self.get_uuid_me()
		if self.uuid_me == '':
			rox.info(_('I don\'t know who you are!\nOpen your card and select My Card->\"That\'s my Card\" from the menu.'))
			return
					
		stack = os.listdir(self.card_folder.value)
		for card in stack:
			if mime.get_type(card, 1) != self.mime_type_vcard:
				continue
			self.get_dict_from_file(os.path.join(self.card_folder.value, card))
			if self.dict.has_key('UID'):
				if self.dict['UID'] == self.uuid_me:
					self.load(os.path.join(self.card_folder.value, card))
					self.init_discard()
					return
		rox.info(_('Sorry, couldn\'t find your card!\nPick a new one!'))
		
	def get_uuid_me(self, widget = None):
		from rox import basedir
		dir = basedir.save_config_path('Contacts')
		try:
			file = open ( dir + '/ME', 'r' )
			self.uuid_me = file.readline()
			file.close()
		except:
			self.uuid_me = ''

	def launch_mailto(self, property):
		app = self.mailto_handler.value
		if app == '': rox.edit_options()
		basename = os.path.basename(app)
		if os.path.isdir(app):
			if os.path.exists(app + "/AppRun"):
				app += '/AppRun'
				L = [app, property.get_text()]
		else:
			if basename == 'sylpheed':
				L = [app, '--compose', property.get_text()]
			elif basename in ['thunderbird', 'mozilla', 'netscape']:
				L = [app, 'mailto:'+property.get_text()]	
			else:
				L = [app, property.get_text()]
		self.command(L)

	def launch_mailto_home_1(self, widget = None):
		self.launch_mailto(self.entry.email_1)

	def launch_mailto_home_2(self, widget = None):
		self.launch_mailto(self.entry.email_2)
		
	def launch_mailto_work(self, widget = None):
		self.launch_mailto(self.entry.work_email)

	def launch_url(self, property):
		app = self.url_handler.value
		if app == '': rox.edit_options()
		basename = os.path.basename(app)
		if os.path.isdir(app):
			if os.path.exists(app + "/AppRun"):
				app += '/AppRun'
				self.command([app, property.get_text()])
		else:
			if basename in [ 'mozilla', 'firefox', 'netscape' ]:
				self.command([app, '--remote', 'openURL("%s", new-window)' % property.get_text()])
			else:
				self.command([app, property.get_text()])

	def launch_url_home(self, widget = None):
		self.launch_url(self.entry.url)
		
	def launch_url_work(self, widget = None):
		self.launch_url(self.entry.work_url)

	def launch_callto(self, property):
		app = self.callto_handler.value
		if app == '': rox.edit_options()
		basename = os.path.basename(app)
		if os.path.isdir(app):
			if os.path.exists(app + "/AppRun"):
				app += '/AppRun'
				self.command([app, property.get_text()])
		else:
			self.command([app, property.get_text()])

	def launch_callto_home(self, widget = None):
		self.launch_callto(self.entry.videoconference)
		
	def launch_callto_work(self, widget = None):
		self.launch_callto(self.entry.work_videoconference)

	
