# -*- coding: UTF8 -*-
## timezone_gui.py - Program creates a user interface 
##             that allows the system time, system date,
##             time zone, and ntpd configuration to be easily set
## Copyright (C) 2001, 2002, 2003 Red Hat, Inc.
## Copyright (C) 2001, 2002, 2003 Brent Fox <bfox@redhat.com>
##                                Tammy Fox <tfox@redhat.com>

## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.

## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.

## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import rox
from rox import g
import gobject
import time
import sys, os

def Dialog(zonetab=rox.app_dir+'/zone.tab', tz=None, geo=None, map=rox.app_dir+"/images/world.jpg"):
    try:
        import worldmap
        dialog = worldmap.TimezoneMap(zonetab, tz, geo, map)
    except:
	    dialog = timezonePage(zonetab, tz, geo)
    return dialog

    
class timezoneBackend:
	
    def getTimezoneInfo (self, tz=None):
        return self.tz

    def setTimezoneInfo (self, timezone):
        self.tz = timezone

    def __init__(self, tz=None):
        if tz and tz[2]:
            self.tz = tz[2]
        else:
            self.tz = "America/New_York"
            path = '/etc/sysconfig/clock'
            lines = []

            if os.access(path, os.R_OK):
                fd = open(path, 'r')
                lines = fd.readlines()
                fd.close()         
        
            try:
                for line in lines:
                    line = line.strip()
                    try:
                        tokens = line.split("=")
                        if tokens[0] == "ZONE":
                            self.tz = tokens[1].replace('"', '')
                    except:
                        pass
            except:
                pass           
            
class Enum:
    def __init__(self, *args):
        i = 0
        for arg in args:
            self.__dict__[arg] = i
            i += 1

class timezonePage (g.VBox, timezoneBackend):
    def __init__(self, zonetab, tz, geo):
        g.VBox.__init__(self, False, 5)
        timezoneBackend.__init__(self, tz)
        
        from zonetab import ZoneTab
        self.zonetab = ZoneTab(zonetab)
        
        self.columns = Enum("TZ", "COMMENTS", "ENTRY")
        
        # set up list of timezones
        self.listStore = g.ListStore(gobject.TYPE_STRING,
                                       gobject.TYPE_STRING,
                                       gobject.TYPE_PYOBJECT)

        self.currentEntry = None
        self.fallbackEntry = None
        self.highlightedEntry = None
        
        for entry in self.zonetab.getEntries():
            iter = self.listStore.append()
            self.listStore.set_value(iter, self.columns.TZ, _(entry.tz))
            if entry.comments:
                self.listStore.set_value(iter, self.columns.COMMENTS,
                                         _(entry.comments))
            else:
                self.listStore.set_value(iter, self.columns.COMMENTS, "")
            self.listStore.set_value(iter, self.columns.ENTRY, entry)
            if entry.tz == self.tz:
                self.currentEntry = entry
            if entry.tz == "America/New_York":
                #In case the /etc/sysconfig/clock is messed up, use New York as default
                self.fallbackEntry = entry     
         
        if geo: (self.currentEntry.lat, self.currentEntry.long) = geo     
                      
        geobox = g.HBox(False)
        self.pack_end(geobox, True, True)
		
        sg = g.SizeGroup(g.SIZE_GROUP_HORIZONTAL)
		
        self.geocheckmark = g.CheckButton()
        self.geocheckmark.set_active(True)
        geobox.pack_start(self.geocheckmark, False, False, 3)
		
        def geo_spinbutton(box, sg, caption, default, minmax):
            label = g.Label(caption+':')
            label.set_alignment(0.5, 0.5)
            box.pack_start(label, False, False, 5)
            button = g.SpinButton(g.Adjustment(default, -minmax, minmax, 1, 1), 1, 0)
            button.set_numeric(False)
            button.set_digits(6)
            box.pack_start(button, False, False, 5)
            sg.add_widget(label)
            return button
            
        def toggle_geo(widget):
            flag = widget.get_active()
            self.latitude.set_sensitive(flag)
            self.longitude.set_sensitive(flag)
	        		
        self.latitude = geo_spinbutton(geobox,sg, _('Latitude'),self.currentEntry.lat,90.0)
        self.longitude = geo_spinbutton(geobox,sg, _('Longitude'),self.currentEntry.long,180.0)
        self.geocheckmark.connect('toggled', toggle_geo)
            
        self.listStore.set_sort_column_id(self.columns.TZ, g.SORT_ASCENDING)

        self.listView = g.TreeView(self.listStore)
        selection = self.listView.get_selection()
        selection.connect("changed", self.selectionChanged)
        self.listView.set_property("headers-visible", False)
        col = g.TreeViewColumn(None, g.CellRendererText(), text=0)
        self.listView.append_column(col)
        col = g.TreeViewColumn(None, g.CellRendererText(), text=1)
        self.listView.append_column(col)

        sw = g.ScrolledWindow ()
        sw.add(self.listView)
        sw.set_shadow_type(g.SHADOW_IN)
        self.pack_end(sw, True, True)
    
        self.setCurrent(self.currentEntry)
        
    def getCurrent(self):
        return self.currentEntry

    def selectionChanged(self, selection, *args):
        (model, iter) = selection.get_selected()
        if iter is None:
            return
        entry = self.listStore.get_value(iter, self.columns.ENTRY)
        self.setCurrent(entry, skipList=1)

    def setCurrent(self, entry, skipList=0):
        if not entry:
            # If the value in /etc/sysconfig/clock is invalid, default to New York
            self.currentEntry = self.fallbackEntry
        else:
            self.currentEntry = entry
            
        self.latitude.set_value(self.currentEntry.lat)
        self.longitude.set_value(self.currentEntry.long)

        if skipList:
            return

        iter = self.listStore.get_iter_first()
        while iter:
            if self.listStore.get_value(iter, self.columns.ENTRY) == self.currentEntry:
                selection = self.listView.get_selection()
                selection.unselect_all()
                selection.select_iter(iter)
                path = self.listStore.get_path(iter)
                col = self.listView.get_column(0)
                self.listView.set_cursor(path, col, False)
                self.listView.scroll_to_cell(path, col, True, 0.5, 0.5)
                break
            iter = self.listStore.iter_next(iter)

 

