# -*- coding: UTF8 -*-
# vim: ts=4 noet

import sys, os, string
import GnuPGInterface

class Key:
	def __init__(self):
		self.keyID = None
		self.UserID = None
		self.value = None
		
def get_value(name):
	"""export key from database"""
	pipe = os.popen('gpg --export --no-secmem-warning --always-trust '+name)
	output = pipe.read()
	pipe.close()
	return output
	
class MyGnuPG(GnuPGInterface.GnuPG):
	
	def __init__(self):
		GnuPGInterface.GnuPG.__init__(self)
		self.setup_my_options()
		
	def setup_my_options(self):
		self.options.armor = 0
		self.options.meta_interactive = 0
		self.options.extra_args.append('--no-secmem-warning')
		self.options.extra_args.append('--always-trust')
		
	def _read_response(self, child_stdout):
		# Internal method: reads all the output from GPG, taking notice
		# only of lines that begin with the magic [GNUPG:] prefix.
		# (See doc/DETAILS in the GPG distribution for info on GPG's
		# output when --status-fd is specified.)
		#
		# Returns a dictionary, mapping GPG's keywords to the arguments
		# for that keyword.

		resp = {}
		while 1:
			line = child_stdout.readline()
			if line == "": break
			line = line.rstrip()
			if line[0:9] == '[GNUPG:] ':
				# Chop off the prefix
				line = line[9:]
				L = line.split(None, 1)
				keyword = L[0]
				if len(L) > 1:
					resp[ keyword ] = L[1]
				else:
					resp[ keyword ] = ""
		return resp
		
		
	def get_armored(self, value):
		"""Extract key ID and User ID"""
		p = self.run(['--armor'], create_fhs=['stdin','stdout'])
		p.handles['stdin'].write(value)
		p.handles['stdin'].close()
		output = p.handles['stdout'].read()
		p.handles['stdout'].close()
	 	
	 	try: p.wait()
		except IOError: pass
		
		return output
		
	def get_value(self, name):
		"""export key from database"""
		p = self.run(['--export'], create_fhs=['stdin', 'stdout'])
		p.handles['stdin'].write(name)
		p.handles['stdin'].close()
		value = p.handles['stdout'].read()
		p.handles['stdout'].close()
		
		try: p.wait()
		except IOError: pass
		
		return value
		 	
	def merge(self, value):
		"""merge/import key"""
		p = self.run(['--import'], create_fhs=['stdin'])
		p.handles['stdin'].write(value)
		p.handles['stdin'].close()
		try: p.wait()
		except IOError: pass

	def get_ID(self, value):
		"""Extract key ID and User ID"""
		p = self.run([], create_fhs=['stdin','stdout'])
		p.handles['stdin'].write(value)
		p.handles['stdin'].close()
		line = p.handles['stdout'].readline()
		p.handles['stdout'].close()
	 	line = line.split('/',1)[1]
	 	l = line.split(' ',2)
	 	
	 	try: p.wait()
		except IOError: pass
		
		return (l[0].strip(), l[2].strip())
	 	
	def get_public_keys(self):
		proc = self.run(['--with-colons', '--list-public-keys'], \
			create_fhs=['stdout'])
		output = proc.handles['stdout'].read()
		proc.handles['stdout'].close()

		keys = {}
		lines = output.split('\n')
		for line in lines:
			sline = line.split(':')
			if sline[0] == 'pub':
				keys[sline[4][8:]] = sline[9]
		
		try: proc.wait()
		except IOError: pass
		
		return keys
		
	def get_secret_keys(self):
		proc = self.run(['--with-colons', '--list-secret-keys'], \
			create_fhs=['stdout'])
		output = proc.handles['stdout'].read()
		proc.handles['stdout'].close()

		keys = {}
		lines = output.split('\n')
		for line in lines:
			sline = line.split(':')
			if sline[0] == 'sec':
				keys[sline[4][8:]] = sline[9]
		
		try: proc.wait()
		except IOError: pass
		
		return keys
			
	def verify(self, str, sign):
		if not str:
			return ''
		from tempfile import TemporaryFile
		file = TemporaryFile(prefix='rox-contacts')
		fd = file.fileno()
		file.write(str)
		file.seek(0)
	
		proc = self.run(['--verify', '--enable-special-filenames', \
						'-', '-&%s'%fd], create_fhs=['stdin', 'status', 'stderr'])

		file.close()
		sign = self.addHeaderFooter(sign, 'SIGNATURE')
		proc.handles['stdin'].write(sign)
		proc.handles['stdin'].close()
		proc.handles['stderr'].close()

		stat = proc.handles['status']
		resp = self._read_response(stat)
		proc.handles['status'].close()

		try: proc.wait()
		except IOError: pass

		keyid = ''
		if resp.has_key('GOODSIG'):
			keyid = resp['GOODSIG'].split()[0]
		elif resp.has_key('BADSIG'):
			keyid = resp['BADSIG'].split()[0]
		return keyid
		
		
	def stripHeaderFooter(self, data):
		"""Remove header and footer from data"""
		lines = data.split('\n')
		while lines[0] != '':
			lines.remove(lines[0])
		while lines[0] == '':
			lines.remove(lines[0])
		i = 0
		for line in lines:
			if line:
				if line[0] == '-': break
			i = i+1
		line = '\n'.join(lines[0:i])
		return line

	def addHeaderFooter(self, data, type):
		"""Add header and footer from data"""
		out = "-----BEGIN PGP %s-----\n" % type
		out = out + "Version: PGP\n"
		out = out + "\n"
		out = out + data + "\n"
		out = out + "-----END PGP %s-----\n" % type
		return out
		
		
			
#print MyGnuPG().get_public_keys()
		
						
	