# -*- coding: UTF8 -*-
# vim: ts=4 noet
# custom widgets for the Contacts GUI
# Copyright (C) 2004 Guido Schimmels

import gc

import rox
from rox import g

def section_header(box, caption):
	label = g.Label('<b>'+caption+'</b>')
	label.set_use_markup(True)
	label.set_alignment(0.01, 0.5)
	box.pack_start(label, False, False, 5)

def entry_with_label(box, sgl, caption, chars=22):
	hbox = g.HBox(False) ; box.pack_start(hbox, True, True, 2)
	label = g.Label(caption+': ') ; label.set_alignment(0.5, 0.5) 
	entry = g.Entry() ; entry.set_activates_default(True)
	entry.set_width_chars(chars)
	hbox.pack_start(label, False, False, 3)
	hbox.pack_start(entry, True, True, 3)
	if sgl != None: sgl.add_widget(label)
	return entry

def comboboxentry_with_label(box, sgl, caption, chars=22):
	hbox = g.HBox(False) ; box.pack_start(hbox, True, True, 0)
	label = g.Label(caption+': ') ; label.set_alignment(0.5, 0.5) 
	combo_box = g.combo_box_entry_new_text()
	entry = combo_box.child
	entry.set_width_chars(chars)
	hbox.pack_start(label, False, False, 3)
	hbox.pack_start(combo_box, True, True, 3)
	if sgl != None: sgl.add_widget(label)
	return combo_box

def option_entry(box, sizegroup, options, selection):
	hbox = g.HBox(False)
	box.pack_start(hbox, True, True, 0)
	option = g.combo_box_new_text()
	sizegroup.add_widget(option)
	for item in options:
		option.append_text(item)
	option.set_active(selection)
	entry = g.Entry()
	entry.set_activates_default(True)
	hbox.pack_start(option, False, False, 3)
	hbox.pack_start(entry, True, True, 3)
	return (entry, option)

def option_entry_with_icon(box, options, selection):
	hbox = g.HBox(False)
	box.pack_start(hbox, False, True, 0)
	liststore = g.ListStore(g.gdk.Pixbuf, str)
	combobox = g.ComboBox(liststore)

	pbcell = g.CellRendererPixbuf()
	combobox.pack_start(pbcell, False)
	combobox.add_attribute(pbcell, 'pixbuf', 0)

	textcell = g.CellRendererText()
	combobox.pack_start(textcell, False)
	combobox.add_attribute(textcell, 'text', 1)
	
	sg = g.SizeGroup(g.SIZE_GROUP_HORIZONTAL)
	sg.add_widget(combobox)
	for item in options:
		icon = g.gdk.pixbuf_new_from_file(rox.app_dir+'/images/im/'+item+'.png')
		liststore.append([icon.scale_simple(16, 16, g.gdk.INTERP_BILINEAR), ' '+item])
		del icon #; gc.collect()
	combobox.set_active(selection)
	entry = g.Entry()
	entry.set_activates_default(True)
	hbox.pack_start(combobox, True, True, 3)
	hbox.pack_start(entry, True, True, 3)
	return (entry, combobox)

def href_button(box, sg, caption,  icon = None):
    hbox = g.HBox(False) ; box.pack_start(hbox, True, True, 0)
    label = g.Label("<span foreground='blue' underline='low'>"+caption+':'+'</span>')
    label.set_use_markup(True)
    button = g.Button()
    button.add(label)
    button.set_relief(g.RELIEF_NONE)
    if icon:
        icon = g.gdk.pixbuf_new_from_file(rox.app_dir+'/images/'+icon)
        scale = icon.scale_simple(16, 16, g.gdk.INTERP_BILINEAR)
        icon = g.Image() ; icon.set_from_pixbuf(scale)
        hbox.pack_start(icon, False, False, 3)
        del icon #; gc.collect()
    sg.add_widget(label)
    entry = g.Entry()
    entry.set_activates_default(True)
    hbox.pack_start(button, False, False, 3)
    hbox.pack_start(entry, True, True, 3)
    return (entry, button)

def edit_button(box, sg, caption,  icon = None):
    hbox = g.HBox(False) ; box.pack_start(hbox, True, True, 1)
    label = g.Label(caption+'...')
    #label.set_use_markup(True)
    button = g.Button()
    button.add(label)
    sg.add_widget(button)
    if icon:
        icon = g.gdk.pixbuf_new_from_file(rox.app_dir+'/images/'+icon)
        scale = icon.scale_simple(16, 16, g.gdk.INTERP_BILINEAR)
        icon = g.Image() ; icon.set_from_pixbuf(scale)
        hbox.pack_start(icon, False, False, 3)
        del icon #; gc.collect()
    entry = g.Entry()
    entry.set_activates_default(True)
    hbox.pack_start(button, False, False, 3)
    hbox.pack_start(entry, True, True, 3)
    return (entry, button)

def date_entry(sg, caption, tips):
	box = g.HBox(False)
	label = g.Label(caption)
	label.set_alignment(0.5, 0.5)
	box.pack_start(label, False, False, 3)
	day = g.SpinButton(g.Adjustment(1, 1, 31, 1, 7), 1, 0)
	day.set_numeric(True)
	box.pack_start(day, False, False, 0)
	month = g.SpinButton(g.Adjustment(1, 1, 12, 1, 3), 1, 0)
	month.set_numeric(True)
	box.pack_start(month, False, False, 0)
	year = g.SpinButton(g.Adjustment(1950, 1900, 2100, 1, 10), 1, 0)
	year.set_numeric(True)
	box.pack_start(year, False, False, 0)
	tips.set_tip(day, _('Day'))
	tips.set_tip(month, _('Month'))
	tips.set_tip(year, _('Year'))
	checked = g.CheckButton()
	box.pack_start(checked, False, False, 0)
	sg.add_widget(label)
	return (box, checked, day, month, year)

