# -*- coding: UTF8 -*-
# timezone_map_gui.py: gui timezone map widget.
#
# Copyright 2001 - 2005 Red Hat, Inc.
#
# This software may be freely redistributed under the terms of the GNU
# library public license.
#
# You should have received a copy of the GNU Library Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#Originally written by Matt Wilson <msw@redhat.com>
#Additions by:
#Brent Fox <bfox@redhat.com>
#Nils Philippsen <nphilipp@redhat.com>

import rox
from rox import g
import gobject
import pango
try:
    import gnomecanvas
except ImportError:
    import gnome.canvas as gnomecanvas
import re
import math
import geodialog

class TimezoneMap(geodialog.timezonePage):
    #force order of destruction for a few items.
    def __del__(self):
        del self.arrow
        del self.markers
        del self.current

    def __init__(self, zonetab, tz, geo, map):
        geodialog.timezonePage.__init__(self, zonetab, tz, geo)
        # set up class member objects
        
        self.markers = {}
        
        # set up the map canvas
        self.canvas = gnomecanvas.Canvas()
        root = self.canvas.root()
        pixbuf = g.gdk.pixbuf_new_from_file(map)
        self.mapWidth = pixbuf.get_width()
        self.mapHeight = pixbuf.get_height()
        root.add(gnomecanvas.CanvasPixbuf, x=0, y=0, pixbuf=pixbuf)
        x1, y1, x2, y2 = root.get_bounds()
        self.canvas.set_scroll_region(x1, y1, x2, y2)
        self.canvas.set_size_request(int(x2), int(y2))
        self.pack_start(self.canvas, False, False)
        
        # set up status bar
        self.status = g.Statusbar()
        self.status.set_has_resize_grip(False)
        self.statusContext = self.status.get_context_id("")
        self.pack_start(self.status, False, False)

        self.current = root.add(gnomecanvas.CanvasText, text='x',
                                fill_color='red', anchor=g.ANCHOR_CENTER,
                                weight=pango.WEIGHT_BOLD)
        
        root.connect("event", self.mapEvent)
        self.canvas.connect("event", self.canvasEvent)
        

        self.arrow = root.add(gnomecanvas.CanvasLine,
                              fill_color='limegreen',
                              width_pixels=2,
                              first_arrowhead=False,
                              last_arrowhead=True,
                              arrow_shape_a=4.0,
                              arrow_shape_b=8.0,
                              arrow_shape_c=4.0,
                              points=(0.0, 0.0, 0.0, 0.0))
        self.arrow.hide()

        for entry in self.zonetab.getEntries():
            x, y = self.map2canvas(entry.lat, entry.long)
            self.markers[entry.tz] = root.add(gnomecanvas.CanvasText, x=x, y=y,
                              text=u'\u00B7', fill_color='yellow',
                              anchor=g.ANCHOR_CENTER,
                              weight=pango.WEIGHT_BOLD)   
                       
        selection = self.listView.get_selection()
        selection.connect("changed", self._selectionChanged)    
        self.setMarker()   
        
    def _selectionChanged(self, selection, *args):
        self.selectionChanged(selection, *args)
        self.setMarker()
        
    def mapEvent(self, widget, event=None):
        if event.type == g.gdk.MOTION_NOTIFY:
            x1, y1 = self.canvas.root().w2i(event.x, event.y)
            lat, long = self.canvas2map(x1, y1)
            last = self.highlightedEntry
            self.highlightedEntry = self.zonetab.findNearest(lat, long)
            if last != self.highlightedEntry:
                self.status.pop(self.statusContext)
                status = _(self.highlightedEntry.tz)
                if self.highlightedEntry.comments:
                    status = "%s - %s" % (status,
                                          _(self.highlightedEntry.comments))
                self.status.push(self.statusContext, status)

            x2, y2 = self.map2canvas(self.highlightedEntry.lat,
                                       self.highlightedEntry.long)
            self.arrow.set(points=(x1, y1, x2, y2))
            self.arrow.show()
        elif event.type == g.gdk.BUTTON_PRESS:
            if event.button == 1:
	            self.setCurrent(self.highlightedEntry)
	            self.setMarker()
                             
    def setMarker(self):
        self.markers[self.currentEntry.tz].show()
        self.markers[self.currentEntry.tz].hide()
        x, y = self.map2canvas(self.currentEntry.lat, self.currentEntry.long)
        self.current.set(x=x, y=y)
                
    def canvasEvent(self, widget, event=None):
        if event.type == g.gdk.LEAVE_NOTIFY:
            self.arrow.hide()
        
    def map2canvas(self, lat, long):
        x2 = self.mapWidth
        y2 = self.mapHeight
        x = x2 / 2.0 + (x2 / 2.0) * long / 180.0
        y = y2 / 2.0 - (y2 / 2.0) * lat / 90.0
        return (x, y)

    def canvas2map(self, x, y):
        x2 = self.mapWidth
        y2 = self.mapHeight
        long = (x - x2 / 2.0) / (x2 / 2.0) * 180.0
        lat = (y2 / 2.0 - y) / (y2 / 2.0) * 90.0
        return (lat, long)
        
